#include "glsphyl_fn.h"
#include "aa_empirical.h"
void getyb(int npair, int nsim, double *v, double *sigma, double *yb){
  int i,j;
  double *yt;
  
  yt = (double *) malloc((size_t) npair*sizeof(double));
  for(i = 0; i < npair; i++) yt[i] = 0;
  rmvnorm(nsim,npair,yt,sigma,yb);
  for(i = 0; i < nsim; i++) 
    for(j = 0; j < npair; j++) yb[j+i*npair] /= sqrt(v[j]);
  free(yt);
}
FILE *fopene(char *filename, char *rw, char *err){
  FILE *fp;
  if((fp=fopen(filename,rw))==NULL){
    printf("%s\n",err);
    exit(1);
  }
  return(fp);
}
char get_param(int *k, char **arg, char *value){
  char opt='z';
  int i;
  
  for(i = *k; i >= 0; i--){
    if(arg[i][0]=='-') opt=arg[i][1];
    if(arg[i][2]!='\0') sprintf(value,"%s",&arg[i][2]);
    if(arg[i][2]=='\0') sprintf(value,"%s",arg[i+1]);
    if(opt!='z'){
      *k=i-1;
      break;
    }
  }
  return(opt);
}
char ctl_paraml(FILE *fp, char *opt, char *value, int *is_option){
  /* return last char on line */
  char c;
  int i;

  *is_option=0;
  while(((c = getc(fp)) == ' ' || c == '\t') && c != EOF && c != '\n');
  if(c == EOF || c == '\n') return(c);
  if(c == '*'){ /* comment */
    while((c = getc(fp)) != '\n' && c != EOF) ;
    return(c);
  } 

  /* option name */
  *is_option=1;
  for(i=0; c !=' ' && c != '\t'; i++){
    opt[i]=c;
    c=getc(fp);
  }
  opt[i] = '\0';
  /* skip '=' */
  while((c=getc(fp))==' ' || c=='\t' || c == '=') ;
  for(i=0; c != ' ' && c != '\t' && c != '\n' && c != EOF; i++){
    value[i]=c;
    c=getc(fp);
  }
  value[i]= '\0';
  while(c != '\n' && c != EOF) c=getc(fp);
  return(c);
}
int pmodel2smodel(int model, char *aaRmat, int nchar){
  if (nchar==4){
    if(model==0) return(0); /* JC */
    if(model==2) return(1); /* F81 */
    if(model==3) return(2); /* F84 */
    if(model==4) return(8); /* HKY */
    if(model==7) return(3); /* GTR */
  }
  if (nchar != 4){
    if(model==0) return(0); /* JC */
    if(model==1) return(1); /* F81 */
    if(model==8) return(3); /* GTR */
  }
  if (*aaRmat!='\0'){
    if(strcmp(aaRmat,"dayhoff.dat")==0) return(4); /* PAM */
    if(strcmp(aaRmat,"jones.dat")==0) return(5); /* JTT */
    if(strcmp(aaRmat,"wag.dat")==0) return(6); /* WAG */
    if(strcmp(aaRmat,"mtREV24.dat")==0) return(7); /* mtREV24 */
    if(strcmp(aaRmat,"lg.dat")==0) return(9); /* LG */
  }
  printf("No substitution model available\n");
  exit(1);
}
int bnewtr(double *a, double *b, double *x, double *f)
{
  const int itmax = 100;
/*   const double tol1 = 1.0e-15, tol2 = 1.0e-9; */
  const double tol1 = 1.0e-6, tol2 = 1.0e-6;
  static int iter = 1;
  static double s;

  if ((*b - *a) < tol2){
    iter = 1;
    return(0);
  }
  s = f[1];
  if (fabs(s) < tol1){
    iter = 1;
    return(0);
  }
  if (f[2] < 0.0)  s = -s/f[2];
  if (s > 0.0){
    *a = *x;
    s = (*x + s > *b) ? 0.5*(*b-*a) : s;
  }
  else{
    *b = *x;
    s = (*x + s < *a) ? 0.5*(*a-*b) : s;
  }
  *x += s;
  if (iter == itmax){
    iter = 1;
    return(0);
  }
  iter++;
  return(1);
}
int l2i(char c){
  if (c == 'a' || c == 'A') return(0);
  if (c == 'c' || c == 'C') return(1);
  if (c == 'g' || c == 'G') return(2);
  if (c == 't' || c == 'T') return(3);
  if (c == '-') return(5);
  return(6);
}
int l2ip(char c){
  switch(c){
  case 'A':
    return(alanine);
    
  case 'R':
    return(arginine);
    
  case 'N':
    return(asparagine);
    
  case 'D':
    return(aspartic);

  case 'C':
    return(cysteine);

  case 'Q':
    return(glutamine);

  case 'E':
    return(glutamic);

  case 'G':
    return(glycine);

  case 'H':
    return(histidine);

  case 'I':
    return(isoleucine);

  case 'L':
    return(leucine);

  case 'K':
    return(lysine);

  case 'M':
    return(methionine);

  case 'F':
    return(phenylalanine);

  case 'P':
    return(proline);

  case 'S':
    return(serine);

  case 'T':
    return(threonine);

  case 'W':
    return(tryptophan);

  case 'Y':
    return(tyrosine);

  case 'V':
    return(valine);

  case '-':
    return(21);
    
  case 'X':
    return(22);

  case '?':
    return(23);
    /* future: cases X and ? represent unknown aa 
     * B - asparagine or aspartic
     * Z - glutamine or glutamic
     * * - stop codon
     */
  }
  return(100);
}
int rseqf(FILE *fp, char *seq){
  int rsite = 0;
  char c;
  
  while((c = getc(fp)) != '\n' &&  c != EOF){
    if (c != ' ' && c != '\r' && c!= '\t'){
      *seq = c;
      seq++;
      rsite++;
    }
  }

  return(rsite);
}
char ignore_whitespace_seqfile(FILE *infile){
  char c;
  while((c = getc(infile)) == ' ' || c == '\n' || c == '\r' || c == '\t') ;
  if(c == EOF){
    printf("seqfile: EOF before entire sequence read");
    exit(1);
  }
  return(c);
}
void rinterleavef(FILE *infile, int *numsp, int *nsite, char name[][11], 
		  char *seq)
{
  int lsite = 0, csite = 0, i, j, initb = 1;
  char c;
  
  fscanf(infile, "%i %i", numsp, nsite); 
  rinterleavef_nolim(infile,numsp,nsite,name,seq);
}
int rinterleave_block(FILE *infile, int numsp, int nsite, char name[][11], 
		       char *seq, int *initb){
  char c;
  int i,j,lsite,lsiteo;
  
  for(i = 0; i < numsp; i++){
    c = ignore_whitespace_seqfile(infile);
    if((*initb) == 1){
      name[i][0] = c;
      for(j = 1; j < 10; j++)name[i][j] = getc(infile);
      name[i][10] = '\0';
      c = ignore_whitespace_seqfile(infile);
    }
    seq[i*nsite] = c;
    lsite = rseqf(infile,&seq[1+i*nsite])+1;
    if(i==0) lsiteo=lsite;

    if(i > 0 && lsite != lsiteo){
      printf("seqfile: number of character states should be same for all taxa in a block\n");
      exit(1);
    }
  }
  *initb = 0;
  
  return(lsite);
}
void rinterleavef_nolim(FILE *infile, const int *numsp, const int *nsite, 
			char name[][11], char *seq)
{
  int lsite = 0, csite = 0, i, j, initb = 1;
  char c;

  while(csite < *nsite){
    lsite = rinterleave_block(infile,*numsp,*nsite,name,&seq[csite],&initb);
    csite += lsite;
  }
}
void letter(int numsp, int nsite, char *seq)
{
  int i, j;
  for(i = 1; i < numsp; i++)
    for(j = 0; j < nsite; j++)
      if (seq[j + i*nsite] == '.') seq[j + i*nsite]=seq[j];
}
int pchar_freqi(int nchar, int nsite, int *s1, int *s2, int *nxy)
{
  int i,n=0;

  for(i = 0; i < nchar*nchar; i++) nxy[i] = 0.0;

  for(i = 0; i < nsite; i++){
    if(s1[i] < nchar && s2[i] < nchar){
      nxy[s1[i]+s2[i]*nchar]++;
      n++;
    }
  }
  return(n);
}
void char_freq(int nchar, int numsp, int nsite, int *seq, double *freq)
{
  int i,j,nc=0;

  for(j = 0; j < nchar; j++){
    freq[j] = 0.0;
  }
  for(i = 0; i < nsite*numsp; i++){
    if(seq[i] < nchar){
      for(j = 0; j < nchar; j++){
	if (seq[i] == j) freq[j]++;
      }
      nc++;
    }
  }

  for(j = 0; j < nchar; j++){
    freq[j] /= nc;
  }
}
int patt_freq(int numsp, int nsite, int *seq, int *count)
{
  int *s;
  int endsite,i,j,k,match;

  s = (int *) malloc((size_t) nsite*numsp*sizeof(int));
  endsite = 0;
  for(i = 0; i < nsite; i++){
    /* check for a match */
    match = 0;
    for(k = 0; k < endsite; k++){
      match = 1;
      for(j = 0; j < numsp; j++){
	if (seq[i+j*nsite] != s[j+k*numsp]){
	  match = 0;
	  break;
	}
      }
      if (match){
	count[k] += 1;
	break;
      }
    }
    
    /* if not a match, new element */
    if (!match){
      for(j = 0; j < numsp; j++) s[j+endsite*numsp] = seq[i+j*nsite];
      count[endsite] = 1;
      endsite++;
    }
  }
  

  for(j = 0; j < numsp; j++)
    for(i = 0; i < endsite; i++)
      seq[i+j*endsite] = s[j+i*numsp];

  free(s);
  return(endsite);
}
int all_sitepatt(int nchar, int ntaxa, int *patt)
{
  /* IN:
   * patt: nchar^ntaxa x ntaxa matrix
   *       each row gives a pattern
   * RETURN: nchar^ntaxa
   *
   * updates patt as 
   *            patt 0
   *            patt 1
   *            ...
   *            patt nchar
   */
  int npatt,npattf,i,j,b,l;
  
  npattf = 1;
  for(i = 0; i < ntaxa; i++) npattf *= nchar;
  for(b = 0; b < nchar; b++) patt[b] = b;
  npatt = nchar;
  for(i = 1; i < ntaxa; i++){

    for(j = 0; j <= i-1; j++)
      for(b = 0; b < nchar; b++)
	for(l = 0; l < npatt; l++){
	  patt[l+b*npatt+j*npattf] = patt[l+j*npattf];
	}

    for(b = 0; b < nchar; b++)
      for(l = 0; l < npatt; l++){ 
	patt[l+b*npatt+i*npattf] = b;
      }

    npatt *= nchar;
  }

  return(npattf);
}
void ou2nu(int numsp, double *outreec, double *utreec)
{
  int *label,i,k,r,l;
  double tmp;

  label = (int *) malloc((size_t) numsp*sizeof(int));
  for(i = 0; i < numsp; i++) label[i] = i;
  for(i = 0; i < (numsp - 1); i++){
    r = (int) outreec[i*4];
    l = (int) outreec[1+i*4];
    utreec[i*4] = label[r];
    utreec[1+i*4] = label[l];
    utreec[2+i*4] = outreec[2+i*4];
    utreec[3+i*4] = outreec[3+i*4];
    label[r] = numsp+i;
    for(k=l; k < (numsp-i-1); k++) label[k]=label[k+1];
    if (utreec[i*4] > utreec[1+i*4]){ /* r < l on return */
      tmp = utreec[i*4]; utreec[i*4] = utreec[1+i*4]; utreec[1+i*4] = tmp;
      tmp = utreec[2+i*4]; utreec[2+i*4] = utreec[3+i*4]; utreec[3+i*4] = tmp;
    }
  }
  
  free(label);
}
void utreec2bl(int ntaxa, double *utreec, double *bl){
  int i,j;
/*   pr_utreec(stdout,ntaxa,utreec); */
  for(i = 0; i < ntaxa-2; i++)
    for(j = 0; j < 2; j++){
/*       printf("%i %f\n",(int) utreec[j+i*4],utreec[j+2+i*4]); */
      bl[(int) utreec[j+i*4]] = utreec[j+2+i*4];
    }
/*   printf("%i %f\n",(int) utreec[(ntaxa-2)*4],utreec[2+(ntaxa-2)*4]); */
  bl[(int) utreec[(ntaxa-2)*4]] = utreec[2+(ntaxa-2)*4]+utreec[3+(ntaxa-2)*4];
}
void bl2utreec(int ntaxa, double *bl, double *utreec){
  int i,j;
  for(i = 0; i < ntaxa-2; i++)
    for(j = 0; j < 2; j++)
      utreec[j+2+i*4] = bl[(int) utreec[j+i*4]];
  utreec[2+(ntaxa-2)*4] = bl[(int) utreec[(ntaxa-2)*4]];
  utreec[3+(ntaxa-2)*4] = 0;
}
void gls_paramf(FILE *fp, FILE **treefile, int *nchar, int *model, 
		char *aaRmat, FILE **Qfile, double *kappa,  
		double *alpha, int *ncatG, int *nrate, FILE **ratefile, 
		FILE **seqfile, int *ntrees, int *neig, 
		int *probwt, int *need_pvalue,
		FILE **covtree, int *nneg, int *simwt, 
		FILE **covfile, FILE **ocovfile)
{
  char c,opt[100],value[100],probwts[100];
  int is_option,done=0;
  *treefile=NULL;
  *Qfile=NULL;
  *ratefile=NULL;
  *seqfile=NULL;
  *aaRmat='\0';
  *covtree=NULL;
  *nchar=4;
  *ncatG=0;
  *alpha=-1;
  *nrate=1;
  *neig=-1;
  *probwt=(*nneg>1)?1:2;
  *need_pvalue=1;
  *probwts='\0';
  *ntrees = 1;
  *nneg=1;
  *simwt=2;
  *covfile=NULL;
  *ocovfile=NULL;
  while(!done){
    c = ctl_paraml(fp,opt,value,&is_option);
    if(is_option){
/*       printf("option: %s value: %s\n",opt, value); */
      if(strcmp(opt,"treefile")==0)
	*treefile=fopene(value,"r","treefile not available");
      if(strcmp(opt,"nchar")==0) *nchar=atoi(value);
      if(strcmp(opt,"model")==0) *model=atoi(value);
      if(strcmp(opt,"aaRatefile")==0) sprintf(aaRmat,"%s",value);
      if(strcmp(opt,"Qfile")==0) 
	*Qfile=fopene(value,"r","Qfile not available");
      if(strcmp(opt,"kappa")==0) *kappa=atof(value);
      if(strcmp(opt,"ttratio")==0) *kappa=-atof(value);
      if(strcmp(opt,"alpha")==0) *alpha=atof(value);
      if(strcmp(opt,"ncatG")==0) *ncatG=atoi(value);
      if(strcmp(opt,"nrate")==0) *nrate=atoi(value);
      if(strcmp(opt,"ratefile")==0)
	*ratefile=fopene(value,"r","ratefile not available");
      if(strcmp(opt,"seqfile")==0) 
	*seqfile=fopene(value,"r","seqfile not available");
      if(strcmp(opt,"ntrees")==0) *ntrees=atoi(value);
      if(strcmp(opt,"neig")==0) *neig=atoi(value);
      if(strcmp(opt,"pvalue")==0) *need_pvalue=atoi(value); 
      if(strcmp(opt,"probwt")==0) *probwt=atoi(value);
      if(strcmp(opt,"pattwt")==0) sprintf(probwts,"%s",value);
      if(strcmp(opt,"covtree")==0) 
	*covtree=fopene(value,"r","covtree file not available");
      if(strcmp(opt,"nneg")==0) *nneg=atoi(value);
      if(strcmp(opt,"simwt")==0) *simwt=atoi(value);
      if(strcmp(opt,"covfile")==0) 
	*covfile=fopene(value,"rb","binary covariance file not available");
      if(strcmp(opt,"ocovfile")==0) 
	*ocovfile=fopen(value,"wb");
    }
    if(c==EOF) done=1;
  }
  if(strcmp(probwts,"observed")==0) {*probwt=0; *simwt=6;}
  if(strcmp(probwts,"single")==0) {*probwt=1; *simwt=7;}
  if(strcmp(probwts,"multiple")==0) {*probwt=2; *simwt=2;}
}
double p3LSp(int nchar, int *s, double *fr, double *M)
{
  int i,j,nchar2;
  double px=0.0,pxt;

  nchar2=nchar*nchar;
  for(j = 0; j < nchar; j++){
    pxt=fr[j];
    for(i = 0; i < 3; i++) pxt *= M[j+s[i]*nchar+i*nchar2];
    px += pxt;
  }
  return(px);
}
double p4LSp(int nchar, int top, int *s, double *fr, double *M)
{
  int j,k,nchar2,a1,a2,a3;
  double pxt,px=0;

  nchar2=nchar*nchar;
  for(j = 0; j < nchar; j++)
    for(k = 0; k < nchar; k++){
      pxt=fr[j]*M[j+s[0]*nchar]*M[j+k*nchar+4*nchar2];
      if(top==0){a1=j; a2=k; a3=k;}
      if(top==1){a1=k; a2=j; a3=k;}
      if(top==2){a1=k; a2=k; a3=j;}
/*       printf("%i: %i %i %i\n",top,a1,a2,a3); */
      pxt *= M[a1+s[1]*nchar+nchar2]*M[a2+s[2]*nchar+2*nchar2]*
	M[a3+s[3]*nchar+3*nchar2];
      px += pxt;
    }
  return(px);
}
double p4LSp_eff(int nchar, int top, int *s, double *fr, double *M)
{
  int j,i,nchar2,t2,t3;
  double px=0,Ma[20],Mb[20];

  nchar2=nchar*nchar;
  for(i = 0; i < nchar; i++) 
    Ma[i] = M[i+s[0]*nchar]*M[i+s[(top+1)]*nchar+(top+1)*nchar2];
  for(i = 0; i < nchar; i++)
    for(Mb[i]=0.0, j=0; j < nchar; j++)
      Mb[i] += M[i+j*nchar+4*nchar2]*Ma[j];
  t2=(top==1)?1:2;
  t3=(top==2)?1:3;
  for(px=0.0, i = 0; i < nchar; i++) 
    px += fr[i]*Mb[i]*M[i+s[t2]*nchar+t2*nchar2]*M[i+s[t3]*nchar+t3*nchar2];
  return(px);
}
void pr3LS(int nchar, int numsp, int *s3, int *jv, double *dm, int smodel, 
	   int nrates, double *rate, double *wt, double alpha, 
	   double *fr, double *W, double *Lam, double *px)
{
  int i,j,id=0,nchar2,npatt,nt,nb,pc[4];
  double y[3],tvo[3],one=1.0,pxt,*M;
  double X[]={1,1,0, 1,0,1, 0,1,1};
  double utreec[]={0,1,0,0, 2,3,0,0};

  /*   int k,l; */

  nchar2=nchar*nchar;
  npatt=nchar2*nchar;
  nt=3;
  nb = 2*nt-3;
  M=(double *) malloc(nb*nchar2*nrates*sizeof(double));
  
  /* edge-lengths through LS estimation  */
  for(i = 0; i < nt; i++)
    for(j = (i+1); j < nt; j++) y[id++]=dm[jv[i]+jv[j]*numsp];
  gls_eig_opt(3,3,y,X,tvo);
  
  /* transition matrices for edges */
  utreec[2]=tvo[0]; utreec[3]=tvo[1]; utreec[2+4]=tvo[2]; 
  transm_rate(smodel,nchar,nt,utreec,nrates,rate,alpha,fr,W,Lam,0,M,&one,&one);
/*   for(j = 0; j < 3; j++) printf("%f\n",tvo[j]); */
/*   printf("%i %f %f\n",nrates, wt[0],rate[0]); */
/*   for(j = 0; j < 3; j++){ */
/*     printf("%i\n",j); */
/*     for(k = 0; k < 4; k++){ */
/*       for(l = 0; l < 4; l++) printf("%.3e ",M[k+l*nchar+j*nchar2]); */
/*       printf("\n"); */
/*     }} */
/*   for(i = 0; i < nchar; i++) printf("%f ",fr[i]); printf("\n"); */
  
  /* probability calculations */
  for(i = 0; i < npatt; i++){
    for(j = 0; j < nt; j++) pc[j]=s3[i+j*npatt];
/*     for(j = 0; j < nt; j++) printf("%2i",pc[j]); printf("\n"); */
    px[i]=0.0;
    for(j = 0; j < nrates; j++){
      pxt=p3LSp(nchar,pc,fr,&M[j*nb*nchar2]);
      px[i] += wt[j]*pxt;
    }
  }
  
  free(M);
}
void pr4LS(int nchar, int numsp, int *s4, int *jv, double *dm, int smodel, 
	   int nrates, double *rate, double *wt, double alpha, 
	   double *fr, double *W, double *Lam, double *px)
{
  int i,j,id,top,topo,nchar2,npatt,nt,nb,pc[4],npair;
  double y[6],tv[5],tvo[5],ss,sso=1.0e+20,one=1.0,pxt,*M;
  double X[]={1,1,1,0,0,0, 1,0,0,1,1,0, 0,1,0,1,0,1, 0,0,1,0,1,1,
	      0,1,1,1,1,0,
	      1,1,1,0,0,0, 1,0,0,1,1,0, 0,1,0,1,0,1, 0,0,1,0,1,1,
	      1,0,1,1,0,1,
	      1,1,1,0,0,0, 1,0,0,1,1,0, 0,1,0,1,0,1, 0,0,1,0,1,1,
	      1,1,0,0,1,1};
  double utreec[]={0,1,0,0, 2,3,0,0, 4,5,0,0};

  nchar2=nchar*nchar;
  npatt=nchar2*nchar2;
  npair=(numsp*(numsp-1))/2;
  nt=4;
  nb = 2*nt-3;
  M=(double *) malloc(nb*nchar2*nrates*sizeof(double));
  
  /* edge-lengths through LS estimation  */
  for(top = 0; top < 3; top++){
    for(id=0, i = 0; i < nt; i++)
      for(j = (i+1); j < nt; j++) y[id++]=dm[jv[i]+jv[j]*numsp];
    ss=gls_eig_opt(6,5,y,&X[top*30],tv);
    if(ss < sso){
      sso=ss; memcpy(tvo,tv,5*sizeof(double)); topo=top;
    }
/*     printf("%i %f\n",top,ss); */
  } 
/*   printf("\n"); */
  
  /* transition matrices for edges */
  utreec[2]=tvo[0]; utreec[3]=tvo[1]; utreec[2+4]=tvo[2]; 
  utreec[3+4]=tvo[3]; utreec[2+2*4]=tvo[4];
  transm_rate(smodel,nchar,nt,utreec,nrates,rate,alpha,fr,W,Lam,0,M,&one,&one);
  
/*   printf("%i%i %i%i\n",jv[0],jv[1],jv[2],jv[3]); */
/*   printf("%i ",topo); */
/*   for(i = 0; i < 5; i++) printf("%f ",tvo[i]); printf("\n"); */
/*   exit(0); */
  
  /* probability calculations */
  for(i = 0; i < npatt; i++){
    for(j = 0; j < nt; j++) pc[j]=s4[i+j*npatt];
    px[i]=0.0;
    for(j = 0; j < nrates; j++){
      pxt=p4LSp(nchar,topo,pc,fr,&M[j*nb*nchar2]);
/*       printf("%f %f\n",wt[j],pxt); */
      px[i] += wt[j]*pxt;
    }
  }
  free(M);
  return;
}
void pr3LStree(int nchar, int *s3, double d01, double d02, double d12, 
	       int smodel, int nrates, double *rate, double *wt, double alpha, 
	       double *fr, double *W, double *Lam, double *px)
{
  int i,j,nchar2,npatt,nt,nb,pc[4];
  double *M,one=1.0,pxt;
  double utreec[]={0,1,0,0, 2,3,0,0};
  
  nchar2=nchar*nchar;
  npatt=nchar2*nchar;
  nt=3;
  nb = 2*nt-3;
  M=(double *) malloc(nb*nchar2*nrates*sizeof(double));
  utreec[2]=(d01+d02-d12)*0.5;
  utreec[3]=(d01+d12-d02)*0.5;
  utreec[2+4]=(d02+d12-d01)*0.5;
  transm_rate(smodel,nchar,nt,utreec,nrates,rate,alpha,fr,W,Lam,0,M,&one,&one);
  
  for(i = 0; i < npatt; i++){
    for(j = 0; j < nt; j++) pc[j]=s3[i+j*npatt];
    /*     for(j = 0; j < nt; j++) printf("%2i",pc[j]); printf("\n"); */
    px[i]=0.0;
    for(j = 0; j < nrates; j++){
      pxt=p3LSp(nchar,pc,fr,&M[j*nb*nchar2]);
      px[i] += wt[j]*pxt;
    }
  }
  free(M);
}
void pr4LStree(int nchar, int *s4, 
	       double d01, double d02, double d03, double d12, 
	       double d13, double d23, int smodel, 
	       int nrates, double *rate, double *wt, double alpha, 
	       double *fr, double *W, double *Lam, double *px)
{
  int i,j,top,nchar2,npatt,nt,nb,pc[4];
  double tv[5],one=1.0,pxt,*M,v01,v02,v12;
  double utreec[]={0,1,0,0, 2,3,0,0, 4,5,0,0};

  nchar2=nchar*nchar;
  npatt=nchar2*nchar2;
  nt=4;
  nb = 2*nt-3;
  M=(double *) malloc(nb*nchar2*nrates*sizeof(double));
  
/*   printf(" %f %f %f %f %f %f\n",d01,d02,d03,d12,d13,d23); */
  v01 = d02+d13-d01-d23;
  v02 = d03+d12-d01-d23;
  v12 = d03+d12-d02-d13;
  if(v01 >= 0 && v02 >= 0){ top=0; tv[4]=v01/2; }
  if(v01 <= 0 && v12 >= 0){ top=1; tv[4]=v12/2; }
  if(v02 <= 0 && v12 <= 0){ top=2; tv[4]=-v02/2; }
  tv[0]=(2*(d01+d02+d03)-d12-d13-d23-2*tv[4])/6.0;
  tv[1]=(2*(d01+d12+d13)-d02-d03-d23-2*tv[4])/6.0;
  tv[2]=(2*(d02+d12+d23)-d01-d03-d13-2*tv[4])/6.0;
  tv[3]=(2*(d03+d13+d23)-d01-d02-d12-2*tv[4])/6.0;
/*   printf("%i ",top); */
/*   printf("%f %f %f ",v01,v02,v12); */
/*   for(i = 0; i < 5; i++) printf("%f ",tv[i]); printf("\n"); */
/*   exit(0); */

  /* transition matrices for edges */
  utreec[2]=tv[0]; utreec[3]=tv[1]; utreec[2+4]=tv[2]; 
  utreec[3+4]=tv[3]; utreec[2+2*4]=tv[4];
  transm_rate(smodel,nchar,nt,utreec,nrates,rate,alpha,fr,W,Lam,0,M,&one,&one);
  
  /* probability calculations */
  for(i = 0; i < npatt; i++){
    for(j = 0; j < nt; j++) pc[j]=s4[i+j*npatt];
    px[i]=0.0;
    for(j = 0; j < nrates; j++){
      pxt=p4LSp(nchar,top,pc,fr,&M[j*nb*nchar2]);
/*       printf("%f %f\n",wt[j],pxt); */
      px[i] += wt[j]*pxt;
    }
  }
  free(M);
  return;
}
void pr4LStree_fprob(int nchar, int top, double *M, double *Ma){
  int s1,s0,c,i,e,j,nchar2;
  nchar2=nchar*nchar;
  for(s1 = 0; s1 < nchar; s1++)
    for(s0 = 0; s0 < nchar; s0++){
      c = s1*nchar+s0; /* cols (0,0), (1,0),...,(nc-1,0),...,(nc-1,nc-1) */
      for(i=0; i < nchar; i++){
	e = i+c*nchar;
	Ma[e] = 0.0;
	for(j=0; j < nchar; j++)
	  Ma[e] += M[i+j*nchar+4*nchar2]*M[j+s0*nchar]*
	    M[j+s1*nchar+(top+1)*nchar2]; /* top + 1 is the neighbor of 0 */
      }}
}
double p4LSp_eff2(int nchar, int top, int t2, int t3, int *s, double *fr, 
		double *M,  double *Ma){
  int i,c,nchar2;
  double pxt=0.0;
  nchar2=nchar*nchar;
  c = s[0]+s[top+1]*nchar;
  for(i = 0; i < nchar; i++)
    pxt += fr[i]*Ma[i+c*nchar]*M[i+s[t2]*nchar+t2*nchar2]*
      M[i+s[t3]*nchar+t3*nchar2];
  return(pxt);
}
void pr4LStree_eff(int nchar, int *s4, 
		   double d01, double d02, double d03, double d12, 
		   double d13, double d23, int smodel, 
		   int nrates, double *rate, double *wt, double alpha, 
		   double *fr, double *W, double *Lam, double *px)
{
  int i,j,top,nchar2,nchar3,nchar4,nt,nb,pc[4],t2,t3;
  double tv[5],one=1.0,pxt,*M,*Ma,v01,v02,v12;
  double utreec[]={0,1,0,0, 2,3,0,0, 4,5,0,0};

  nchar2=nchar*nchar;
  nchar3=nchar*nchar2;
  nchar4=nchar*nchar3;
  nt=4;
  nb = 2*nt-3;
  M=(double *) malloc(nb*nchar2*nrates*sizeof(double));
  Ma=(double *) malloc(nchar3*nrates*sizeof(double));
  
  v01 = d02+d13-d01-d23;
  v02 = d03+d12-d01-d23;
  v12 = d03+d12-d02-d13;
  if(v01 >= 0 && v02 >= 0){ top=0; tv[4]=v01/2; }
  if(v01 <= 0 && v12 >= 0){ top=1; tv[4]=v12/2; }
  if(v02 <= 0 && v12 <= 0){ top=2; tv[4]=-v02/2; }
  tv[0]=(2*(d01+d02+d03)-d12-d13-d23-2*tv[4])/6.0;
  tv[1]=(2*(d01+d12+d13)-d02-d03-d23-2*tv[4])/6.0;
  tv[2]=(2*(d02+d12+d23)-d01-d03-d13-2*tv[4])/6.0;
  tv[3]=(2*(d03+d13+d23)-d01-d02-d12-2*tv[4])/6.0;

  /* transition matrices for edges */
  utreec[2]=tv[0]; utreec[3]=tv[1]; utreec[2+4]=tv[2]; 
  utreec[3+4]=tv[3]; utreec[2+2*4]=tv[4];
  transm_rate(smodel,nchar,nt,utreec,nrates,rate,alpha,fr,W,Lam,0,M,&one,&one);

  for(j=0; j < nrates; j++) 
    pr4LStree_fprob(nchar,top,&M[j*nb*nchar2],&Ma[j*nchar3]);
  
  /* topology is (0,top+1),(t2,t3) */
  t2=(top==1)?1:2;
  t3=(top==2)?1:3;
  
  /* probability calculations */
  for(i = 0; i < nchar4; i++){
    for(j = 0; j < nt; j++) pc[j]=s4[i+j*nchar4];
    px[i]=0.0;
    for(j = 0; j < nrates; j++){
      pxt=p4LSp_eff2(nchar,top,t2,t3,pc,fr,&M[j*nb*nchar2],&Ma[j*nchar3]);
/*       printf("%f %f\n",wt[j],pxt); */
      px[i] += wt[j]*pxt;
    }
  }
  free(M); free(Ma);
  return;
}
void tree2dmat(int numsp, double *dmat, double *utreec, double *dmatt, 
	       double minedge)
{
  int i,j,k,*Xi,nb,npair,idx=0;
  double *X,*y,*tv;

  nb=2*numsp-3;
  npair=((numsp-1)*numsp)/2;

  X = (double *) malloc((size_t) npair*nb*sizeof(double));
  y = (double *) malloc((size_t) npair*sizeof(double));
  tv = (double *) malloc((size_t) nb*sizeof(double));
  Xi = (int *) malloc((size_t) npair*nb*sizeof(int));

  create_Xmat(numsp, utreec, Xi);
  for(i = 0; i < npair*nb; i++) X[i]=Xi[i];
  gety(numsp, dmat, y);
/*   for(i = 0; i < npair; i++){ */
/*     for(j = 0; j < nb; j++) printf("%2i",Xi[i+j*npair]); */
/*     printf(" %.3f\n",y[i]); */
/*   } */
  gls_eig_opt(npair, nb, y, X, tv);
  for(i = 0; i < nb; i++) if(tv[i] < minedge) tv[i]=minedge;
  Xb2utreec(numsp,Xi,tv,utreec);
/*   pr_utreec(stdout,numsp,utreec); */
/*   exit(0); */
/*   for(i = 0; i < nb; i++) printf("%.3f ",tv[i]); printf("\n"); */

  for(i = 0; i < numsp*numsp; i++) dmatt[i]=0.0;
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      for(k = 0; k < nb; k++) 
	dmatt[i+j*numsp] += Xi[idx+k*npair]*tv[k];

      dmatt[j+i*numsp] = dmatt[i+j*numsp];
      idx++;
    }
  free(X); free(y); free(tv); free(Xi); 
}
double utreecdpair(int r, int l, int ntaxa, double *utreec){
  int join,k,j,p[2];
  double d=0.0;

  p[0]=r; p[1]=l;
  if(p[0]==p[1]) return(0.0);
  for(join = 0; join < ntaxa-1; join++){
    for(k = 0; k < 2; k++)
      for(j = 0; j < 2; j++)
	if(utreec[k+join*4] == p[j]){
	  d += utreec[k+2+join*4];
	  p[j] = join+ntaxa;
	}
    if(p[1]==p[0]) return(d);
  }
  return(d);
}
void utreec2dmat(int ntaxa, double *utreec, double *dmat){
  int i,j;
  
  for(i = 0; i < ntaxa*ntaxa; i++) dmat[i]=0.0;
  for(i = 0; i < ntaxa; i++)
    for(j = (i+1); j < ntaxa; j++){
      dmat[i+j*ntaxa] = utreecdpair(i,j,ntaxa,utreec);
      dmat[j+i*ntaxa] = dmat[i+j*ntaxa];
    }
}
void sigma_calc(int nsite, int *ngap, int npair, double *sigma, double *fp, 
		double *fpp, int pdef)
{
  int i,j,k,ni;
  double *ipi;

  ipi = (double *) malloc((size_t) npair*sizeof(double));
/*     for(k = 0; k < nsite; k++){ */
/*       for(i = 0; i < npair; i++){ */
/*         printf("%e ", fp[k+i*nsite]); */
/*       } */
/*       printf("\n"); */
/*     } */
/*     exit(1); */
/*   for(i = 0; i < npair*nsite; i++) ngap[i]=1; */
  for(i = 0; i < npair; i++){
    for(j = 0; j < npair; j++){ /* E[l_{ki}'l_ik'] */
      sigma[i+j*npair] = 0.0;
      for(k = 0; k < nsite; k++)
	if(ngap[k+i*nsite] && ngap[k+j*nsite]){
	  sigma[i+j*npair] += fp[k+i*nsite]*fp[k+j*nsite];
	}
/*       printf("% .5e\n",sigma[i+j*npair]); */
    }}
/*   exit(0); */
  
  for(i = 0; i < npair; i++){ /* E[-l_{ki}'']**(-1) */
    ipi[i] = 0.0;
    ni=0;
    for(k = 0; k < nsite; k++)
      if(ngap[k+i*nsite]){
	ipi[i] -= fpp[k + i*nsite];
	ni++;
      }
/*     printf("%i\n",ni); */
    ipi[i] /= ni;
    ipi[i] = 1.0/ipi[i];
    ipi[i] /= ni;
  }
/*    for(i = 0; i < npair; i++) printf("%.16e\n", ipi[i]); */
/*    exit(1); */


  for(i = 0; i < npair; i++){
    for(j = 0; j < npair; j++){
      /* two asymptotic equivalents, the first is more likely to be positive definite */
      if (pdef)
        sigma[i+j*npair] = ipi[i]*ipi[j]*sigma[i+j*npair];
      else
        if (i == j)
	  sigma[i+j*npair] = ipi[i];
        else
	  sigma[i+j*npair] = ipi[i]*ipi[j]*sigma[i+j*npair];
    }
  }
  free(ipi);
}
void dpair(int nchar, int nsite, int *s1, int *s2, int nrate, double *prob, 
	   double *M, double *Mp, double *Mpp, int *ngap, double *fp, 
	   double *fpp)
{
  int i,j,nchar2;
  double f[3],ip;

  nchar2=nchar*nchar;
  ip=0.0;
  for(i = 0; i < nsite; i++){
    if(s1[i] < nchar && s2[i] < nchar){ 
      for(j = 0; j < 3; j++) f[j] = 0;
      for(j = 0; j < nrate; j++){
	f[0] += prob[j]*M[s1[i]+s2[i]*nchar+j*nchar2];
	f[1] += prob[j]*Mp[s1[i]+s2[i]*nchar+j*nchar2];
	f[2] += prob[j]*Mpp[s1[i]+s2[i]*nchar+j*nchar2];
      }
      fp[i] = f[1]/f[0];
      fpp[i] = f[2]/f[0] - (f[1]/f[0])*(f[1]/f[0]);
/*       printf("%i %i % .5e\n",s1[i],s2[i],fpp[i]); */
      ip += fpp[i];
      ngap[i]=1;
    }
    else{
      fp[i] = 0;
      fpp[i] = 0;
      ngap[i] = 0;
    }
  }
/*   printf("%.6e\n",ip); */
/*   printf("%.6e\n",-nsite/ip); */
/*   exit(0); */
  return;
}
double dpair_pr(int nchar, int nsite, int *nxy, int nrate, double *prob, 
		double *fr, double *M, double *Mp, double *fp, int pr)
{
  int j,s1,s2,nchar2;
  double f[3],px,E2=0;

  nchar2=nchar*nchar;
/*   printf("\n"); */
  for(s2 = 0; s2 < nchar; s2++)
    for(s1 = 0; s1 < nchar; s1++){ 
      for(j = 0; j < 3; j++) f[j] = 0;
      for(j = 0; j < nrate; j++){
	f[0] += prob[j]*fr[s1]*M[s1+s2*nchar+j*nchar2];
	f[1] += prob[j]*fr[s1]*Mp[s1+s2*nchar+j*nchar2];
      }
      if(pr && f[0]==0) 
	fp[s1+s2*nchar] = 0.0;
      else
	fp[s1+s2*nchar] = f[1]/f[0];
      if(pr) px = f[0];
      if(!pr) px = ((double) nxy[s1+s2*nchar])/nsite;
      if(px > 0)
	E2 += px*fp[s1+s2*nchar]*fp[s1+s2*nchar];
/*       printf("%i %i %f %f\n", s1, s2, px, fp[s1+s2*nchar]); */
    }
  E2 = 1.0/(E2*nsite);
  return(E2);
}
double dpair_var(int nchar, int nsite, int *nxy, int nrate, double *prob, 
		 double *fr, double *M, double *Mp, int pr){
  int j,s1,s2,nchar2;
  double f[3],fp,px,E2=0;

  nchar2=nchar*nchar;
  for(s2 = 0; s2 < nchar; s2++)
    for(s1 = 0; s1 < nchar; s1++){ 
      for(j = 0; j < 3; j++) f[j] = 0;
      for(j = 0; j < nrate; j++){
	f[0] += prob[j]*fr[s1]*M[s1+s2*nchar+j*nchar2];
	f[1] += prob[j]*fr[s1]*Mp[s1+s2*nchar+j*nchar2];
      }
      if(pr && f[0]==0) 
	fp = 0.0;
      else
	fp = f[1]/f[0];
      if(pr) px = f[0];
      if(!pr) px = ((double) nxy[s1+s2*nchar])/nsite;
      if(px > 0)
	E2 += px*fp*fp;
    }
  E2 = 1.0/(E2*nsite);
  return(E2);
}
void dnacov(int nchar, int nsite, int numsp, int *seq, int nrate, double *prob, 
	    double *M, double *Mp, double *Mpp,  
	    double *sigma, int pdef)
{

  int nchar2,npair,i,j,idx=0,*ngap;
  double *fpp,*fp;

  nchar2=nchar*nchar;
  npair = (numsp * (numsp - 1))/2;
  fpp = (double *) malloc((size_t) nsite*npair*sizeof(double));
  fp = (double *) malloc((size_t) nsite*npair*sizeof(double));
  ngap = (int *) malloc((size_t) nsite*npair*sizeof(int));
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      dpair(nchar,nsite,&seq[i*nsite],&seq[j*nsite],nrate,prob,
	    &M[idx*nchar2*nrate],&Mp[idx*nchar2*nrate],
	    &Mpp[idx*nchar2*nrate],&ngap[idx*nsite],
	    &fp[idx*nsite],&fpp[idx*nsite]);
      idx++;
    }
  sigma_calc(nsite, ngap, npair, sigma, fp, fpp, pdef);
  free(fpp); free(fp); free(ngap);
}
void  new_pair(int n, int *i, int *j)
{
  if(*j<n) 
    (*j)++;
  else{
    (*i)++;
    *j=*i+1;
  }
}
void dnacov_fr(int nchar, int nsite, int numsp, int *seq, int *nxy, int *nv,
	       int nrate, double *prob, double *fr, double *M, double *Mp, 
	       double *sigma)
{
  int nchar2,npair,i,j,idx=0,id[4],p1,p2,*s4,pc[4],nc,*count,ngp;
  double E2,*fp;

/*   printf("dnacov with frequencies or probabilities of patterns\n"); */
  nchar2=nchar*nchar;
  npair = (numsp * (numsp - 1))/2;
  fp = (double *) malloc((size_t) nchar2*npair*sizeof(double));
  s4 = (int *) malloc((size_t) 4*nsite*sizeof(int));
  nc = nchar2*nchar2;
  if(nsite < nc) nc=nsite;
  count = (int *) malloc((size_t) nc*sizeof(int));

  for(i = 0; i < npair*npair; i++) sigma[i]=0;
  
  idx=0;
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      E2=dpair_pr(nchar,nv[idx],&nxy[idx*nchar2],nrate,prob,fr,
		 &M[idx*nchar2*nrate],&Mp[idx*nchar2*nrate],&fp[idx*nchar2],0);
      sigma[idx+idx*npair] = E2;
      idx++;
    }
/*   for(i = 0; i < npair; i++) printf("%.5e\n",Vjj[i]); */
/*   for(i = 0; i < npair; i++) printf("%.5e\n",sigma[i+i*npair]); */
/*   for(i = 0; i < npair; i++) printf("%i\n",nv[i]); */
/*   exit(0); */

  id[0]=0; id[1]=0;
  for(p1 = 0; p1 < npair; p1++){ /* first pair */
    new_pair(numsp-1,id,&id[1]);
    id[2]=id[0]; id[3]=id[1];
    for(p2 = p1+1; p2 < npair; p2++){ /* second pair */
      new_pair(numsp-1,&id[2],&id[3]);
/*       for(i = 0; i < 4; i++) printf("%i ",id[i]); printf("\n"); */

      for(i = 0; i < 4; i++)
	memcpy(&s4[i*nsite],&seq[id[i]*nsite],nsite*sizeof(int));
      nc=patt_freq(4,nsite,s4,count);

      for(i = 0; i < nc; i++){
	for(j = 0; j < 4; j++) pc[j]= s4[i+j*nc];
	ngp=1;
	for(j = 0; j < 4; j++) 
	  if(pc[j] > nchar){ 
	    ngp=0;
	    break;
	  }
	if(ngp){
	  sigma[p1+p2*npair] += count[i]*fp[pc[0]+pc[1]*nchar+p1*nchar2]*
	    fp[pc[2]+pc[3]*nchar+p2*nchar2];
	}
      }
/*       j=0; */
/*       for(i = 0; i < nc; i++) j += count[i]; */
      sigma[p1+p2*npair] *= sigma[p1+p1*npair]*sigma[p2+p2*npair];
      sigma[p2+p1*npair] = sigma[p1+p2*npair];
    }
  }
  free(fp); free(s4); free(count);
  return;
}
void dnacov_pr(int nchar, int nsite, int numsp, int *seq, int *nxy, int *nv,
	       int nrate, double *rate, double *prob, double alpha,
	       double *fr, int smodel, double *W, double *Lam, double *dm,
	       double *M, double *Mp, double *Mpp, double *sigma)
{
  int nchar2,npair,i,j,idx=0,id[4],jv[4],p1,p2,*s4,*s3,*s,npatt,npatt3;
  int npatt4,pc[4],np12,nt;
  double E2,*fp,*px;

/*   printf("dnacov with frequencies or probabilities of patterns\n"); */
  nchar2=nchar*nchar;
  npair = (numsp * (numsp - 1))/2;
  fp = (double *) malloc((size_t) nchar2*npair*sizeof(double));
  s3 = (int *) malloc((size_t) nchar*nchar2*4*sizeof(int));
  s4 = (int *) malloc((size_t) nchar2*nchar2*4*sizeof(int));
  px = (double *) malloc((size_t) nchar2*nchar2*sizeof(double));
/*   printf("%i %i %i\n",nsite,nchar2*nchar2,nc); */

  for(i = 0; i < npair*npair; i++) sigma[i]=0;
  
  idx=0;
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      E2=dpair_pr(nchar,nv[idx],&nxy[idx*nchar2],nrate,prob,fr,
		 &M[idx*nchar2*nrate],&Mp[idx*nchar2*nrate],&fp[idx*nchar2],1);
      sigma[idx+idx*npair] = E2;
      idx++;
    }

/*   for(i = 0; i < npair; i++) printf("%.5e\n",sigma[i+i*npair]); */
/*   exit(0); */

  npatt3=all_sitepatt(nchar,3,s3);
  npatt4=all_sitepatt(nchar,4,s4);

  id[0]=0; id[1]=0;
  for(p1 = 0; p1 < npair; p1++){ /* first pair */
    new_pair(numsp-1,id,&id[1]);
    id[2]=id[0]; id[3]=id[1];
    for(p2 = p1+1; p2 < npair; p2++){ /* second pair */
      new_pair(numsp-1,&id[2],&id[3]);
/*       for(i = 0; i < 4; i++) printf("%i",id[i]); printf("\n"); */

      /* number of non-gapped sites */
      for(np12=0, i = 0; i < nsite; i++)
	if(seq[i+id[0]*nsite] < nchar && seq[i+id[1]*nsite] < nchar 
	   && seq[i+id[2]*nsite] < nchar && seq[i+id[3]*nsite] < nchar) 
	  np12++;

      nt=4; /* 3 or 4 taxa? */
      memcpy(jv,id,4*sizeof(int));
      if(id[2]==id[0] || id[2]==id[1]){ jv[2]=id[3]; nt=3; }
      if(id[3]==id[1]){ jv[2]=id[2]; nt=3; }

      if(nt==3)
	pr3LS(nchar,numsp,s3,jv,dm,smodel,nrate,rate,prob,alpha,fr,W,Lam,px);
      if(nt==4)
	pr4LS(nchar,numsp,s4,jv,dm,smodel,nrate,rate,prob,alpha,fr,W,Lam,px);

      npatt=(nt==3)?npatt3:npatt4;
      s=(nt==3)?s3:s4;
      
      for(i = 0; i < npatt; i++){
	for(j = 0; j < nt; j++) pc[j]= s[i+j*npatt];
	if(nt==3){
	  if(id[2]==id[0]){pc[2]=pc[0]; pc[3]=s[i+2*npatt];}
	  if(id[2]==id[1]){pc[2]=pc[1]; pc[3]=s[i+2*npatt];}
	  if(id[3]==id[1]){pc[3]=pc[1]; pc[2]=s[i+2*npatt];}
	}
	sigma[p1+p2*npair] += px[i]*fp[pc[0]+pc[1]*nchar+p1*nchar2]*
	  fp[pc[2]+pc[3]*nchar+p2*nchar2];
      }
      sigma[p1+p2*npair] *= np12*sigma[p1+p1*npair]*sigma[p2+p2*npair];
      sigma[p2+p1*npair] = sigma[p1+p2*npair];
    }
  }
  free(fp); free(s3); free(s4); free(px);
  return;
}
void dnacov_prtree(int nchar, int nsite, int numsp, int *seq, int *nxy, int *nv,
		   int nrate, double *rate, double *prob, double alpha,
		   double *fr, int smodel, double *W, double *Lam, double *dm,
		   double *sigma)
{
  int nchar2,npair,i,j,idx=0,id[4],jv[4],p1,p2,*s4,*s3,*s,npatt,npatt3;
  int npatt4,pc[4],np12,nt,nmin;
  double E2,*fp,*px,*M,*Mp,utreec[4],no_use;

  /* FILE *binfile; */

  nchar2=nchar*nchar;
  npair = (numsp * (numsp - 1))/2;
  fp = (double *) malloc((size_t) nchar2*npair*sizeof(double));
  s3 = (int *) malloc((size_t) nchar*nchar2*4*sizeof(int));
  s4 = (int *) malloc((size_t) nchar2*nchar2*4*sizeof(int));
  px = (double *) malloc((size_t) nchar2*nchar2*sizeof(double));
  M = (double *) malloc((size_t) npair*nchar2*nrate*sizeof(double));
  Mp = (double *) malloc((size_t) npair*nchar2*nrate*sizeof(double));

  idx=0;
  for(i = 0; i < numsp; i++)
    for(j = i+1; j < numsp; j++){
      utreec[0]=0; utreec[2]=dm[i+j*numsp];
      transm_rate(smodel,nchar,2,utreec,nrate,rate,alpha,fr,W,Lam,1,
  		  &M[idx*nchar2*nrate],&Mp[idx*nchar2*nrate],&no_use);
      idx++;
    }

  /* binfile = fopen("px.bin","wb"); */

  for(i = 0; i < npair*npair; i++) sigma[i]=0;
  idx=0;
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      E2=dpair_pr(nchar,nv[idx],&nxy[idx*nchar2],nrate,prob,fr,
		 &M[idx*nchar2*nrate],&Mp[idx*nchar2*nrate],&fp[idx*nchar2],1);
      sigma[idx+idx*npair] = E2 * nv[idx];

      /* for(p1 = 0; p1 < nchar; p1++) */
      /* 	for(p2 = 0; p2 < nchar; p2++){ */
      /* 	  for(E2=0.0, l = 0; l < nrate; l++) */
      /* 	    E2 += prob[l]*fr[p1]*M[p1+p2*nchar+l*nchar2+idx*nchar2*nrate]; */
      /* 	  fwrite(&E2,sizeof(double),1,binfile); */
      /* 	} */
      idx++;
    }
  /* exit(0); */


  for(nmin=nv[0], i=1; i<npair; i++) if(nv[i] < nmin) nmin=nv[i];

  npatt3=all_sitepatt(nchar,3,s3);
  npatt4=all_sitepatt(nchar,4,s4);

  id[0]=0; id[1]=0;
  for(p1 = 0; p1 < npair; p1++){ /* first pair */
    new_pair(numsp-1,id,&id[1]);
    id[2]=id[0]; id[3]=id[1];
    for(p2 = p1+1; p2 < npair; p2++){ /* second pair */
      new_pair(numsp-1,&id[2],&id[3]);
      /* for(i = 0; i < 4; i++) printf("%i ",id[i]); printf("\n"); */

      /* number of non-gapped sites */
      for(np12=0, i = 0; i < nsite; i++)
	if(seq[i+id[0]*nsite] < nchar && seq[i+id[1]*nsite] < nchar 
	   && seq[i+id[2]*nsite] < nchar && seq[i+id[3]*nsite] < nchar) 
	  np12++;
      if(np12 < nmin) nmin=np12;

      nt=4; /* 3 or 4 taxa? */
      memcpy(jv,id,4*sizeof(int));
      if(id[2]==id[0] || id[2]==id[1]){ jv[2]=id[3]; nt=3; }
      if(id[3]==id[1]){ jv[2]=id[2]; nt=3; }

      if(nt==3){
	pr3LStree(nchar,s3,dm[jv[0]+jv[1]*numsp],dm[jv[0]+jv[2]*numsp],
		  dm[jv[1]+jv[2]*numsp],smodel,nrate,rate,prob,alpha,
		  fr,W,Lam,px);
	/* for(i = 0; i < npatt3; i++) printf("%f\n",px[i]); */
	/* exit(0); */
      }
      if(nt==4)
	pr4LStree_eff(nchar,s4,
		      dm[id[0]+id[1]*numsp],
		      dm[id[0]+id[2]*numsp],
		      dm[id[0]+id[3]*numsp],
		      dm[id[1]+id[2]*numsp],
		      dm[id[1]+id[3]*numsp],
		      dm[id[2]+id[3]*numsp],smodel,nrate,rate,prob,alpha,
		      fr,W,Lam,px);

      npatt=(nt==3)?npatt3:npatt4;
      s=(nt==3)?s3:s4;

      /* fwrite(px,sizeof(double),npatt,binfile); */
      
      for(i = 0; i < npatt; i++){
	for(j = 0; j < nt; j++) pc[j]= s[i+j*npatt];
	if(nt==3){
	  if(id[2]==id[0]){pc[2]=pc[0]; pc[3]=s[i+2*npatt];}
	  if(id[2]==id[1]){pc[2]=pc[1]; pc[3]=s[i+2*npatt];}
	  if(id[3]==id[1]){pc[3]=pc[1]; pc[2]=s[i+2*npatt];}
	}
/* 	if(p1==59 && p2==77){ */
/* 	  printf("%.5e %.5e %.5e\n",px[i],fp[pc[0]+pc[1]*nchar+p1*nchar2],fp[pc[2]+pc[3]*nchar+p2*nchar2]); */
/* 	} */
/* 	if(px[i] > 0) */
	  sigma[p1+p2*npair] += px[i]*fp[pc[0]+pc[1]*nchar+p1*nchar2]*
	    fp[pc[2]+pc[3]*nchar+p2*nchar2];
      }
      /* sigma[p1+p2*npair] *= np12*sigma[p1+p1*npair]*sigma[p2+p2*npair]; */
      sigma[p1+p2*npair] *= sigma[p1+p1*npair]*sigma[p2+p2*npair];
      sigma[p2+p1*npair] = sigma[p1+p2*npair];
    }
  }
  for(i = 0; i < npair*npair; i++) sigma[i] /= nmin;
  /* fclose(binfile); */
  free(fp); free(s3); free(s4); free(px); free(M); free(Mp); 
  return;
}
void covmldistf(int nchar, int nsite, int numsp, 
		int *seq, int *nxy, int *nv, 
		int nrates, double *rate, double *prob, double alpha, 
		double *fr, int smodel, double *W, double *Lam, 
		double *M, double *Mp, double *Mpp,
		double *dmat, int probwt, FILE *covtree, char (*name)[11],
		double *sigma, double *U, double *Root){
  int npair;
  double *dmatt,*Offdiag,*utreec,minedge=1.0e-12;

  /* clock_t cstart, end; */
  /* double cpu_time_used; */
  /* cstart = clock(); */

  /* char tstr[10000]; */
  
  npair = (numsp * (numsp - 1))/2;
  if(probwt==1 || probwt==7){
    dmatt = (double *) malloc((size_t) numsp*numsp*sizeof(double));
    utreec = (double *) malloc((size_t) (numsp-1)*4*sizeof(double));
  }
  if(probwt <= 5)
    Offdiag = (double *) malloc((size_t) npair*sizeof(double));
  
  if(probwt==0 || probwt==6){
    dnacov_fr(nchar,nsite,numsp,seq,nxy,nv,nrates,prob,fr,M,Mp,sigma);
  }
  if(probwt==1 || probwt==7){
    if(covtree == NULL){
      memcpy(dmatt,dmat,numsp*numsp*sizeof(double));
      nju(numsp,dmat,utreec);
      memcpy(dmat,dmatt,numsp*numsp*sizeof(double));
      tree2dmat(numsp,dmat,utreec,dmatt,minedge);
      /* ut2n(numsp,utreec,name,tstr); */
      /* printf("%s\n",tstr); */
    }
    else{
      read_treecsnl(covtree,numsp,name,utreec,1);
/*       pr_utreec(stdout,numsp,utreec); */
/*       exit(0); */
      utreec2dmat(numsp,utreec,dmatt);
    }
/*       wmat(stdout,numsp,numsp,dmat); */
/*       wmat(stdout,numsp,numsp,dmatt); */
/*       exit(0); */
    
    dnacov_prtree(nchar,nsite,numsp,seq,nxy,nv,nrates,rate,prob,alpha,fr,
		  smodel,W,Lam,dmatt,sigma);
/*     wmat(stdout,npair,npair,sigma); */
  }
  if(probwt==3){
    dnacov_pr(nchar,nsite,numsp,seq,nxy,nv,nrates,rate,prob,alpha,fr,smodel,
	      W,Lam,dmat,M,Mp,Mpp,sigma);
  }

  if(probwt < 4){
    memcpy(U,sigma,npair*npair*sizeof(double));
    eigenRealSym(U, npair, Root, Offdiag);
  }
  
  if(probwt==5){
    dnacov(nchar,nsite,numsp,seq,nrates,prob,M,Mp,Mpp,sigma,0); 
    memcpy(U,sigma,npair*npair*sizeof(double));
    eigenRealSym(U, npair, Root, Offdiag);
    /* probwt=0: try asymptotically equivalent version if singular */
    memcpy(U,sigma,npair*npair*sizeof(double));
    eigenRealSym(U, npair, Root, Offdiag);
/*     if(Root[npair-1] <= 0.0){ */
/*       dnacov(nchar,nsite,numsp,seq,nrates,prob,M,Mp,Mpp,U,1); */
/*       memcpy(sigma,U,npair*npair*sizeof(double)); */
/*       eigenRealSym(U, npair, Root, Offdiag); */
/*       if (Root[npair-1] <= 0.0){ */
/* 	printf("glsdna: V likely singular\nexiting: try glsdna_eig\n"); */
/* 	exit(1); */
/*       } */
/*     } */
  }

  if(probwt==1 || probwt==7){
    free(dmatt); 
    free(utreec);
  }
  if(probwt <= 5) free(Offdiag);

  /* end = clock(); */
  /* cpu_time_used = ((double) (end - cstart)) / CLOCKS_PER_SEC; */
  /* printf("CPU time: %f\n",cpu_time_used); */
  return;
}
void varmldistf(int nchar, int numsp, int *nxy, int *nv, int nrate, 
		   double *prob, double *fr, double *M, double *Mp, 
		   int probwt, double *sigma){
  /* probwt should be 0 or 1 */
  int i,j,nchar2,idx=0;
  nchar2=nchar*nchar;
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      sigma[idx] = dpair_var(nchar,nv[idx],&nxy[idx*nchar2],nrate,prob,
			      fr,&M[idx*nchar2*nrate],
			      &Mp[idx*nchar2*nrate],probwt);
      idx++;
    }
}
void varmldistf_tree(int nchar, int numsp, int *nxy, int *nv, int nrate, 
		     double *rate, double *prob, double alpha, double *fr, 
		     int smodel, double *W, double *Lam, double *dmatt, 
		     double *sigma){
  int nchar2,npair,idx,i,j;
  double *M,*Mp,utreec[4],no_use;

  nchar2=nchar*nchar;
  npair = (numsp * (numsp - 1))/2;
  M = (double *) malloc((size_t) npair*nchar2*nrate*sizeof(double));
  Mp = (double *) malloc((size_t) npair*nchar2*nrate*sizeof(double));
  
  idx=0; 
  for(i = 0; i < numsp; i++)
    for(j = i+1; j < numsp; j++){
      utreec[0]=0; utreec[2]=dmatt[i + j*numsp];
      transm_rate(smodel,nchar,2,utreec,nrate,rate,alpha,fr,W,Lam,1,
		  &M[idx*nchar2*nrate],&Mp[idx*nchar2*nrate],&no_use);
      idx++;
    }
  varmldistf(nchar,numsp,nxy,nv,nrate,prob,fr,M,Mp,1,sigma);
  free(M); free(Mp);
  return;
}
void wlssim(int ntaxa, double *y, double *sigma, int nchar, int smodel, 
	    double *fr, double *W, double *Lam, 
	    int nrate, double *rate, double *prob, double alpha,
	    int *nxy, int *nv, int probwt, int nsim, double *yb, double *v){
  int npair,isim,idx,p1,p2,nchar2;
  double utreec[4],one=1.0,*M,*Mp;

  npair = (ntaxa*(ntaxa-1))/2;
  nchar2 = nchar*nchar;
  probwt -= 6; 
  M = (double *) malloc((size_t) npair*nchar2*nrate*sizeof(double));
  Mp = (double *) malloc((size_t) npair*nchar2*nrate*sizeof(double));
  for(isim = 0; isim < nsim; isim++){
    idx=0;
    for(p1 = 0; p1 < ntaxa; p1++)
      for(p2 = p1+1; p2 < ntaxa; p2++){
	utreec[2]=yb[idx+isim*npair];
	transm_rate(smodel,nchar,2,utreec,nrate,rate,alpha,fr,W,Lam,1,
		    &M[idx*nchar2*nrate],&Mp[idx*nchar2*nrate],&one);
	idx++;
      }
    varmldistf(nchar,ntaxa,nxy,nv,nrate,prob,fr,M,Mp,probwt,&v[isim*npair]);
  }
  free(M); free(Mp);
  return;
}
double wlssimp(int nsim, int npair, int xcol, double ssobs, double *yb, 
	       double *Q){
  int isim,i,j;
  double yt,ss,pval=0.0;
  for(isim = 0; isim < nsim; isim++){
    ss = 0.0;
    for(i = 0; i < (npair-xcol); i++){
      yt = 0.0;
      for(j = 0; j < npair; j++) yt += Q[j+i*npair]*yb[j+isim*npair];
      ss += yt*yt;
    }
/*     printf("%f %f\n",ss,ssobs); */
    if(ss > ssobs) pval += 1.0;
  }
  return(pval/nsim);
}
void gety(int numsp, double *d, double *y)
{
  int i, j, idx;

  idx = 0;
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      y[idx] = d[i+j*numsp];
      idx++;
    }
}
void scaly_eig(int npe, int npair, double *Root, double *U, double *y, 
	       double *yt)
{
  int i, j;

  for(i = 0; i < npe; i++){
    yt[i] = 0.0;
    for(j = 0; j < npair; j++){
      yt[i] += U[i+j*npair]*y[j];
    }
    yt[i] /= sqrt(Root[i]);
  }
  for(i = 0; i < npe; i++) y[i] = yt[i];
}
void v_trans(int numsp, int *v, int *x)
{
  /* IN: v, numsp vector of 0 or 1 gives a split
   * OUT: x column of X corresp to v 
   * (i,j) are on opposite sides of split if vi=0,vj=1 or vj=0,vi=1, 
   *    <-> vi*(1-vj)+vj*(1-vi)=1 */
  int i, j, idx;
  idx = 0;
  for(i = 0; i < (numsp-1); i++)
    for(j = (i+1); j < numsp; j++){
      x[idx] = v[i]*(1-v[j]) + v[j]*(1-v[i]);
      idx++;
    }
}
void update_v(int numsp, int idx, int *v, double *uvec)
{
  int k, r, l;
  r = (int) uvec[0]; l = (int) uvec[1];
  for(k = 0; k < numsp; k++)
    if(v[k+r*numsp] == 1 || v[k+l*numsp] == 1) v[k+idx*numsp] = 1;
}
void create_Xmat(int numsp, double *utreec, int *X)
{
  int i,npair, r;
  int *v;
  
  v = (int *) malloc(2*numsp*numsp*sizeof(int)); 

/*   int j; */
  
  npair = (numsp*(numsp-1))/2;
  for(i = 0; i < numsp*(2*numsp-3); i++) v[i]=0;/* initialize v */
  for(i = 0; i < numsp; i++) v[i+i*numsp] = 1; /* terminal splits */

  for(i = 0; i < numsp; i++) /* first numsp cols of X */
    v_trans(numsp, &v[i*numsp], &X[i*npair]);

  for(i = 0; i < (numsp - 3); i++){
    /* i+numsp th column of v gives the ith split with descendants =1*/
    update_v(numsp, i+numsp, v, &utreec[i*4]);
    r = utreec[i*4];
    v_trans(numsp, &v[(i+numsp)*numsp], &X[numsp*npair+i*npair]);
  }
/*   for(i = 0; i < numsp; i++){ */
/*     for(j = 0; j < 2*numsp-3; j++){ */
/*       printf("%2i",v[i+j*numsp]); */
/*     } */
/*     printf("\n"); */
/*   } */
  free(v);
}
void getXmat(char *tstring, int numsp, char name[][11], int *X)
{
  double *utreec;
/*   int i; */
  utreec = (double *) malloc((numsp-1)*4*sizeof(double));

/*   printf("%s\n", tstring); */
/*   for(i = 0; i < numsp; i++) printf("%s\n",name[i]); */
  treecsnl(numsp, tstring, name, utreec, 1);  /* obtain the utreec matrix */
/*    for(i = 0; i < numsp-1; i++){ */
/*       printf("%i %i %f %f\n", (int) utreec[i*4], (int) utreec[1+ i*4],  utreec[2+ i*4],  utreec[3+ i*4]); */
/*    } */
  create_Xmat(numsp, utreec, X);
  free(utreec);
}
void scalX_eig(int npair, int npe, int xcol, int *Xi, double *Root, double *U, 
	       double *X)
{
  int i, j, k;
  
  for(j = 0; j < xcol; j++){
      for(i = 0; i < npe; i++){
	X[i+j*npe] = 0.0;
	for(k = 0; k < npair; k++){
	  X[i+j*npe] += U[i+k*npair]*Xi[k+j*npair];
	}
	X[i+j*npe] /= sqrt(Root[i]);
      }
    }
}
void X2splits(int numsp, int *Xi, int *s){
  int i,j,npair;
  npair=((numsp-1)*numsp)/2;
  for(i = 0; i < numsp*(numsp-3); i++) s[i]=1;
  for(i = 0; i < numsp-3; i++)
    for(j = 0; j < numsp-1; j++)
      if(Xi[j+(i+numsp)*npair]==1) s[j+1+i*numsp] = 0;
}
void Xb2utreec(int numsp, int *Xi, double *beta, double *utreec){
  int i,*tree,*s;
/*   int j; */
  s=(int *) malloc((numsp-3)*numsp*sizeof(int));
  tree=(int *) malloc(numsp*sizeof(int));
  X2splits(numsp,Xi,s);
/*   for(i = 0; i < numsp-3; i++){ */
/*     printf("%i: ",i); */
/*     for(j = 0; j < numsp; j++) printf("%i",s[j+i*numsp]); */
/*     printf("\n"); */
/*   } */
  for(i = 0; i < numsp-3; i++) tree[i]=i;
  splits2tree_bl(numsp,numsp-3,tree,s,beta,utreec);
  free(tree); free(s);
}
void read_trees(FILE *treefile, int ntrees, char *tstring, int *start)
{
  int idx, ltr;

  start[0] = 0;
  for(idx = 0; idx < ntrees; idx++){
    ltr = read_tree(treefile, &tstring[start[idx]]);
    start[idx+1] = start[idx] + ltr;
  }
}
double gls_eig_opt(int np, int xcol, double *y, double *X, double *beta)
{
  int *index, mode; 
  double rnorm, *w, *zz; 

  index = (int *) malloc(np*sizeof(int));
  w = (double *) malloc(xcol*sizeof(double));
  zz = (double *) malloc(np*sizeof(double));

  nnls_(X, &np, &np, &xcol, y, beta, &rnorm, w, zz, index, &mode);  

  free(index);
  free(w);
  free(zz);
  return(rnorm*rnorm);
}
void makeprotfreqs(double *fr, double *W, double *Lam) 
{
  long i, mineig,nchar=20;
  mineig = 0;
  for (i = 0; i < nchar; i++)
    if (fabs(Lam[i]) < fabs(Lam[mineig]))
      mineig = i;
  for(i = 0; i < nchar; i++) fr[i] =  fabs(W[i+mineig*nchar]);
}
void chQ_freq(int nbin, double *W, double *Lam, double *fr, double *fro)
{
  /* binning/udistfbf.c 
   *     Q = Pi^(-1) W Lam W' 
   * decomposition used to obtain P(t) as
   *     P(t) = Pi^(-1) W exp(Lam t) W'
   * where W is determined from eigen-decomp
   *     Pi^(1/2) Q Pi^(-1/2) = U' Lam U,   W = (Pi)^(1/2) U'
   * To adjust for frequencies Pi*:
   * obtain Q_ij* = Q_ij pi_j* / pi_j
   *        Q_ii* = - sum_{j neq j} Q_ij*
   *        M_ij* =  sqrt(pi_i*) Q_ij* / sqrt(pi_j*)
   *        U and Lam from M = U' Lam U (eigen-decomp)
   *        W = (Pi*)^(1/2) U' */
  int i,j,l;
  double R[400], Offdiag[400], Rt, mu;

/*   for(i = 0; i < nbin; i++) */
/*     for(j = 0; j < nbin; j++){ */
/*       Rt = 0.0; */
/*       for(l = 0; l < nbin; l++) */
/* 	Rt += W[i+l*nbin]*W[j+l*nbin]*Lam[l]; */
/*       Rt /= fro[i]; */
/*       if (i != j) */
/* 	printf("%f\n", Rt); */
/*     } */
/*   exit(0); */

  for(i = 0; i < nbin; i++){ /* R_ij = Q_ij / (pi_i pi_j) */
    for(j = (i+1); j < nbin; j++){
      Rt = 0.0;
      for(l = 0; l < nbin; l++){
	Rt += W[i+l*nbin]*W[j+l*nbin]*Lam[l];
      }
      Rt /= (fro[i]*fro[j]);
      R[i+j*nbin] = Rt; R[j+i*nbin] = Rt;
    }
  }

  for(i = 0; i < nbin; i++){ /* R_ii = Q_ii* */
    Rt = 0.0;
    for(j = 0; j < nbin; j++){
      if (j != i)
	Rt -= R[i+j*nbin]*fr[j];
    }
    R[i+i*nbin] = Rt/fr[i]; /* will cancel with fr[i] below */
  }

/*   for(i = 0; i < nbin; i++) */
/*     for(j = 0; j < nbin; j++) printf("%f\n",R[i+j*nbin]*fr[j]); exit(0); */

  for(i = 0; i < nbin; i++){ /* R_ij = entry of M_ij* above */
    for(j = i; j < nbin; j++){
      R[i+j*nbin] *= sqrt(fr[i]*fr[j]);
      if (j != i) R[j+i*nbin] = R[i+j*nbin];
    }
  }

  eigenRealSym(R, nbin, Lam, Offdiag); /* eigen-decomp of M* */
  for(i = 0; i < nbin; i++){ /* W = (Pi*)^(1/2) U' */
    for(j = 0; j < nbin; j++){
      W[i+j*nbin] = R[j+i*nbin] * sqrt(fr[i]);
    }
  }

  /* rescale eigenvalues so that -sum_i pi_i Q_ii = 1 */
  mu = 0.0;
  for(i = 0; i < nbin; i++)
    for(l = 0; l < nbin; l++) mu -= Lam[l]*W[i+l*nbin]*W[i+l*nbin];
  for(i = 0; i < nbin; i++) Lam[i] /= mu;
}
double int_elamrt(double t, double lam, double alpha, int deriv, double *de){
  double te,dt;
  te = -lam*t/alpha;
  dt= exp(-alpha*log(1+te));
  if(deriv==0) return(dt);
  if(deriv==1 || deriv==2){
    de[0]=lam*exp(-(alpha+1)*log(1+te));
    if(deriv==2) de[1]=(alpha+1)*lam*lam*exp(-(alpha+2)*log(1+te))/alpha;
  }
  if(deriv== 3 || deriv==4){
    de[0] = dt*(te/(1+te)-log(1+te));
    if(deriv==4) 
      de[1]=de[0]*de[0]/dt + 
	dt*(te*te/(alpha*(1+te)*(1+te))-te/(alpha*(1+te)));
  }
  return(dt);
}
void pJCm(double t, int nchar, double *fr, int deriv, 
	    double *M, double *Mp, double *Mpp){
  int i,j;
  double m,emt;
  m = nchar/(nchar - 1.0);
  emt=exp(-t*m);
  for(i = 0; i < nchar; i++)
    for(j = 0; j < nchar; j++){
      if(i==j){
	M[i+j*nchar]=1.0/nchar+(1.0-1.0/nchar)*emt;
	if(deriv==1 || deriv==2) Mp[i+j*nchar]=-m*(1.0-1.0/nchar)*emt;
	if(deriv==2) Mpp[i+j*nchar]=m*m*(1.0-1.0/nchar)*emt;
      }
      if(i!=j){
	M[i+j*nchar]=1.0/nchar*(1.0-exp(-m*t));
	if(deriv==1 || deriv==2) Mp[i+j*nchar]=m*1.0/nchar*emt;
	if(deriv==2) Mpp[i+j*nchar]=-m*m*1.0/nchar*emt;
      }
    }
}
void pJCm_gen(double t, int nchar, double *fr, double *W, double *Lam, 
		int deriv, double *M, double *Mp, double *Mpp){
  pJCm(t,nchar,fr,deriv,M,Mp,Mpp);
}
void pJCma(double t, int nchar, double *fr, int deriv, double alpha,
	   double *M, double *Mp, double *Mpp){
  int i,j;
  double mu,emut,demu[2];
  mu = nchar/(nchar - 1.0);
  emut=int_elamrt(t,-1.0*mu,alpha,deriv,demu);

  for(i = 0; i < nchar; i++)
    for(j = 0; j < nchar; j++){
      if(i==j){
	M[i+j*nchar]=1.0/nchar+(1.0-1.0/nchar)*emut;
	if(deriv > 0) 
	  Mp[i+j*nchar]=(1.0-1.0/nchar)*demu[0];
	if(deriv==2 || deriv==4) Mpp[i+j*nchar]=(1.0-1.0/nchar)*demu[1];
      }
      if(i!=j){
	M[i+j*nchar]=1.0/nchar-emut/nchar;
	if(deriv > 0) 
	  Mp[i+j*nchar]=-demu[0]/nchar;
	if(deriv==2 || deriv==4) Mpp[i+j*nchar]=-demu[1]/nchar;
      }
    }
}
void pJCma_gen(double t, int nchar, double *fr, double *W, double *Lam, 
	       int deriv, double alpha, double *M, double *Mp, double *Mpp){
  pJCma(t,nchar,fr,deriv,alpha,M,Mp,Mpp);
}
double muF81f(int nchar, double *fr){
  int l;
  double mu = 0.0;
  for(l = 0; l < nchar; l++) mu += fr[l]*(1.0-fr[l]);
  return(1.0/mu);
}
void pF81m(double t, int nchar, double *fr, double mu, int deriv, 
	   double *M, double *Mp, double *Mpp){
  int i,j;
  double emut;
  emut=exp(-mu*t);
  for(i = 0; i < nchar; i++)
    for(j = 0; j < nchar; j++){
      if(i==j){
	M[i+j*nchar]=fr[j]+(1.0-fr[j])*emut;
	if(deriv==1 || deriv==2) Mp[i+j*nchar]=-mu*(1.0-fr[j])*emut;
	if(deriv==2) Mpp[i+j*nchar]=mu*mu*(1.0-fr[j])*emut;
      }
      if(i!=j){
	M[i+j*nchar]=fr[j]*(1.0-exp(-mu*t));
	if(deriv==1 || deriv==2) Mp[i+j*nchar]=mu*fr[j]*emut;
	if(deriv==2) Mpp[i+j*nchar]=-mu*mu*fr[j]*emut;
      }
    }
}
void pF81m_gen(double t, int nchar, double *fr, double *mu, double *Lam, 
		 int deriv, double *M, double *Mp, double *Mpp){
  pF81m(t,nchar,fr,*mu,deriv,M,Mp,Mpp);
}
void pF81ma(double t, int nchar, double *fr, double mu, int deriv, double alpha,
	   double *M, double *Mp, double *Mpp){
  int i,j;
  double emut,demu[2];
  emut=int_elamrt(t,-1.0*mu,alpha,deriv,demu);
  for(i = 0; i < nchar; i++)
    for(j = 0; j < nchar; j++){
      if(i==j){
	M[i+j*nchar]=fr[j]+(1.0-fr[j])*emut;
	if(deriv > 0) Mp[i+j*nchar]=(1-fr[j])*demu[0];
	if(deriv==2 || deriv==4) Mpp[i+j*nchar]=(1-fr[j])*demu[1];
      }
      if(i!=j){
	M[i+j*nchar]=fr[j]-fr[j]*emut;
	if(deriv > 0) Mp[i+j*nchar]=-fr[j]*demu[0];
	if(deriv==2 || deriv==4) Mpp[i+j*nchar]=-fr[j]*demu[1];
      }
    }
}
void pF81ma_gen(double t, int nchar, double *fr, double *mu, double *Lam, 
		int deriv, double alpha, double *M, double *Mp, double *Mpp){
  pF81ma(t,nchar,fr,*mu,deriv,alpha,M,Mp,Mpp);
}
int is_transitionf(int i, int j){
  return((i==0 && j==2) || (i==2 && j==0) || (i==1 && j==3) || (i==3 && j==1));
}
double muF84f(double K, double *fr){
  double mu = 0.0;
  int l;
/*   printf("%f\n",K); */
  for(l = 0; l < 4; l++) mu += fr[l]*(1.0-fr[l]);
  mu += 2*K*( fr[0]*fr[2]/(fr[0]+fr[2]) + fr[1]*fr[3]/(fr[1]+fr[3]) );
  return(1.0/mu);
}
void pF84m(double t, double mu, double K, double *fr, int deriv, 
	   double *M, double *Mp, double *Mpp){
  int i,j;
  double Pr,Py,Pj,muK,emuKt,emut;
  Pr=fr[0]+fr[2];
  Py=fr[1]+fr[3];
  muK=mu*(K+1); emuKt=exp(-muK*t); emut=exp(-mu*t);

  for(j = 0; j < 4; j++){
    if (j == 0 || j == 2) Pj=Pr;
    if (j == 1 || j == 3) Pj=Py;
    for(i = 0; i < 4; i++){
      if(i==j){
	M[i+j*4]=fr[j]+fr[j]*(1/Pj-1)*emut+(1-fr[j]/Pj)*emuKt;
	if(deriv==1 || deriv==2)
	  Mp[i+j*4]=-mu*fr[j]*(1/Pj-1)*emut-muK*(1-fr[j]/Pj)*emuKt;
	if(deriv==2)
	  Mpp[i+j*4]=mu*mu*fr[j]*(1/Pj-1)*emut + muK*muK*(1-fr[j]/Pj)*emuKt;
      }
      if(i!=j){
	if(is_transitionf(i,j)){
	  M[i+j*4]=fr[j]+fr[j]*(1/Pj-1)*emut-(fr[j]/Pj)*emuKt;
	  if(deriv==1 || deriv==2)
	    Mp[i+j*4]=-mu*fr[j]*(1/Pj-1)*emut+muK*(fr[j]/Pj)*emuKt;
	  if(deriv==2)
	    Mpp[i+j*4]=mu*mu*fr[j]*(1/Pj-1)*emut-muK*muK*(fr[j]/Pj)*emuKt;
	}
	else{
	  M[i+j*4]=fr[j]*(1.0-emut);
	  if(deriv==1 || deriv==2) Mp[i+j*4] = mu*fr[j]*emut;
	  if(deriv==2) Mpp[i+j*4]=-mu*mu*fr[j]*emut;
	}
      }
    }
  }
}
void pF84m_gen(double t, int nchar, double *fr, double *mu, double *K, 
	       int deriv, double *M, double *Mp, double *Mpp){
  pF84m(t,*mu,*K,fr,deriv,M,Mp,Mpp);
}
void pF84ma(double t, double mu, double K, double *fr, int deriv, double alpha,
	    double *M, double *Mp, double *Mpp){
  int i,j;
  double Pr,Py,Pj,emuKt,emut,demuK[2],demu[2];
  Pr=fr[0]+fr[2];
  Py=fr[1]+fr[3];
  emut=int_elamrt(t,-1.0*mu,alpha,deriv,demu);
  emuKt=int_elamrt(t,-1.0*mu*(K+1),alpha,deriv,demuK);
/*   if (deriv==4) printf("%f %f\n",emut,emuKt); */

  for(j = 0; j < 4; j++){
    if (j == 0 || j == 2) Pj=Pr;
    if (j == 1 || j == 3) Pj=Py;
    for(i = 0; i < 4; i++){
      if(i==j){
	M[i+j*4]=fr[j]+fr[j]*(1/Pj-1)*emut+(1-fr[j]/Pj)*emuKt;
	if(deriv > 0)
	  Mp[i+j*4]=fr[j]*(1/Pj-1)*demu[0]+(1-fr[j]/Pj)*demuK[0];
	if(deriv==2 || deriv==4)
	  Mpp[i+j*4]=fr[j]*(1/Pj-1)*demu[1]+(1-fr[j]/Pj)*demuK[1];
      }
      if(i!=j){
	if(is_transitionf(i,j)){
	  M[i+j*4]=fr[j]+fr[j]*(1/Pj-1)*emut-(fr[j]/Pj)*emuKt;
	  if(deriv > 0)
	    Mp[i+j*4]=fr[j]*(1/Pj-1)*demu[0]-(fr[j]/Pj)*demuK[0];
	  if(deriv==2 || deriv==4)
	    Mpp[i+j*4]=fr[j]*(1/Pj-1)*demu[1]-(fr[j]/Pj)*demuK[1];
	}
	else{
	  M[i+j*4]=fr[j]-fr[j]*emut;
	  if(deriv > 0) Mp[i+j*4] = -fr[j]*demu[0];
	  if(deriv==2 || deriv==4) Mpp[i+j*4]=-fr[j]*demu[1];
	}
      }
    }
  }
}
void pF84ma_gen(double t, int nchar, double *fr, double *mu, double *K, 
		int deriv, double alpha, double *M, double *Mp, double *Mpp){
  pF84ma(t,*mu,*K,fr,deriv,alpha,M,Mp,Mpp);
}
double tt2K(double R, double *fr){
  double pY,pR,K;
  pY=fr[1]+fr[3]; pR=fr[0]+fr[2];
  K=(R*pR*pY-fr[0]*fr[2]-fr[1]*fr[3])/(fr[0]*fr[2]/pR+fr[1]*fr[3]/pY);
  return(K);
}
double K2tt(double K, double *fr){
  double pY,pR,R;
  pY=fr[1]+fr[3]; pR=fr[0]+fr[2];
  R=K*(fr[0]*fr[2]/pR+fr[1]*fr[3]/pY)/(pR*pY) + 
    (fr[0]*fr[2]+fr[1]*fr[3])/(pR*pY);
  return(R);
}
double muHKYf(double kappa, double *fr){
  double mu;
  mu=2.0*(kappa*(fr[0]*fr[2]+fr[1]*fr[3])+(fr[0]+fr[2])*(fr[1]+fr[3]));
  return(1.0/mu);
}
void pHKYm(double t, int nchar, double *fr, double mu, double kappa,
	   int deriv, double *M, double *Mp, double *Mpp){
  int i,j;
  double Pr,Py,A,emuAt,emut,muA,Pj;

  Pr=fr[0]+fr[2];
  Py=fr[1]+fr[3];
  emut=exp(-mu*t);

  for(j = 0; j < 4; j++){

    if (j == 0 || j == 2) Pj=Pr;
    if (j == 1 || j == 3) Pj=Py;
    A = 1+Pj*(kappa-1); muA=mu*A;
    emuAt=exp(-muA*t);

    for(i = 0; i < 4; i++){
      if(i==j){
	M[i+j*4] = fr[j]+fr[j]*(1/Pj-1)*emut+(1-fr[j]/Pj)*emuAt;
	if(deriv==1 || deriv==2)
	  Mp[i+j*4] = -mu*fr[j]*(1/Pj-1)*emut-muA*(1-fr[j]/Pj)*emuAt;
	if(deriv==2)
	  Mpp[i+j*4] = mu*mu*fr[j]*(1/Pj-1)*emut+muA*muA*(1-fr[j]/Pj)*emuAt;
      }
      if(i!=j){
	if(is_transitionf(i,j)){
	  M[i+j*4] = fr[j]+fr[j]*(1/Pj-1)*emut-fr[j]*emuAt/Pj;
	  if(deriv==1 || deriv==2)
	    Mp[i+j*4] = -mu*fr[j]*(1/Pj-1)*emut+muA*fr[j]*emuAt/Pj;
	  if(deriv==2)
	    Mpp[i+j*4] = mu*mu*fr[j]*(1/Pj-1)*emut-muA*muA*fr[j]*emuAt/Pj;
	}
	else{
	  M[i+j*4] = fr[j]-fr[j]*emut;
	  if(deriv==1 || deriv==2) Mp[i+j*4] = mu*fr[j]*emut;
	  if(deriv==2) Mpp[i+j*4] = -mu*mu*fr[j]*emut;
	}
      }
    }}
}
void pHKYm_gen(double t, int nchar, double *fr, double *mu, double *kappa, 
	       int deriv, double *M, double *Mp, double *Mpp){
  pHKYm(t,nchar,fr,*mu,*kappa,deriv,M,Mp,Mpp);
}
void pHKYma(double t, int nchar, double *fr, double mu, double kappa,
	    int deriv, double alpha, double *M, double *Mp, double *Mpp){
  int i,j;
  double Pr,Py,A,emuAt,demuA[2],emut,demu[2],muA,Pj;

  Pr=fr[0]+fr[2];
  Py=fr[1]+fr[3];
  emut=int_elamrt(t,-1.0*mu,alpha,deriv,demu);

  for(j = 0; j < 4; j++){

    if (j == 0 || j == 2) Pj=Pr;
    if (j == 1 || j == 3) Pj=Py;
    A = 1+Pj*(kappa-1); muA=mu*A;
    emuAt=int_elamrt(t,-1.0*muA,alpha,deriv,demuA);

    for(i = 0; i < 4; i++){
      if(i==j){
	M[i+j*4] = fr[j]+fr[j]*(1/Pj-1)*emut+(1-fr[j]/Pj)*emuAt;
	if(deriv > 0)
	  Mp[i+j*4] = fr[j]*(1/Pj-1)*demu[0]+(1-fr[j]/Pj)*demuA[0];
	if(deriv==2 || deriv==4)
	  Mpp[i+j*4] = fr[j]*(1/Pj-1)*demu[1]+(1-fr[j]/Pj)*demuA[1];
      }
      if(i!=j){
	if(is_transitionf(i,j)){
	  M[i+j*4] = fr[j]+fr[j]*(1/Pj-1)*emut-fr[j]*emuAt/Pj;
	  if(deriv > 0)
	    Mp[i+j*4] = fr[j]*(1/Pj-1)*demu[0]-fr[j]*demuA[0]/Pj;
	  if(deriv==2 || deriv==4)
	    Mpp[i+j*4] = fr[j]*(1/Pj-1)*demu[1]-fr[j]*demuA[1]/Pj;
	}
	else{
	  M[i+j*4] = fr[j]-fr[j]*emut;
	  if(deriv > 0) Mp[i+j*4] = -fr[j]*demu[0];
	  if(deriv==2 || deriv==4) Mpp[i+j*4] = -fr[j]*demu[1];
	}
      }
    }}
}
void pHKYma_gen(double t, int nchar, double *fr, double *mu, double *kappa, 
	       int deriv, double alpha, double *M, double *Mp, double *Mpp){
  pHKYma(t,nchar,fr,*mu,*kappa,deriv,alpha,M,Mp,Mpp);
}
double tt2kappa(double R, double *fr){
  double pR,pY;
  pR=fr[0]+fr[2];
  pY=fr[1]+fr[3];
  return(R*pR*pY/(fr[0]*fr[2]+fr[1]*fr[3]));
}
double kappa2tt(double kappa,  double *fr){
  double pR,pY;
  pR=fr[0]+fr[2];
  pY=fr[1]+fr[3];
  return(kappa*(fr[0]*fr[2]+fr[1]*fr[3])/(pR*pY));
}
void rmnegpv(int m, double *pv, int incp){
  int i,mi;
  double sum=0.0;
  mi=m*incp;
  for(i=0; i<mi; i+=incp)
    if(pv[i] < 0.0)
      pv[i] = 0.0;
    else
      sum += pv[i];
  for(i=0; i<mi; i+=incp) pv[i] /= sum;
}
void rmnegtrans(int m, double *P){
  int i,j;

  for(i=0; i<m; i++)
    for(j=0; j<m; j++)
      if(P[i+j*m] < 0.0){
	rmnegpv(m,&P[i],m);
	break;
      }
  return;
}
void Q2frWLam(int nbin, double *Q, double *fr, double *W, double *Lam){
  /*     Q = Pi^(-1) W Lam W' 
   *  decomposition used to obtain P(t) as
   *     P(t) = Pi^(-1) W exp(Lam t) W'
   * where W is determined from eigen-decomp
   *     Pi^(1/2) Q Pi^(-1/2) = U' Lam U,   W = (Pi)^(1/2) U' 
   * Assumes Q corresponds to GTR model
   */
  int i,j;
  double R[400], Offdiag[400], mu;

  /* pi_j = [1 + sum_i neq j Q_ji/Q_ij]**(-1) */
  for(j = 0; j < nbin; j++){
    fr[j] = 1.0;
    for(i = 0; i < nbin; i++)
      if (j != i) fr[j] += Q[j+i*nbin]/Q[i+j*nbin];
    fr[j] = 1.0 / fr[j];
  }
  for(i = 0; i < nbin; i++)
    for(j = 0; j < nbin; j++) /* R =  Pi^(1/2) Q Pi^(-1/2) */
      R[i+j*nbin] = sqrt(fr[i])*Q[i+j*nbin]/sqrt(fr[j]);

  /* eigen-decomp of Pi^(1/2) Q Pi^(-1/2) */
  eigenRealSym(R, nbin, Lam, Offdiag); 

  for(i = 0; i < nbin; i++){ /* W = (Pi*)^(1/2) U' */
    for(j = 0; j < nbin; j++)
      W[i+j*nbin] = R[j+i*nbin] * sqrt(fr[i]);
  }

  /* rescale eigenvalues so that -sum_i pi_i Q_ii = 1 */
  mu = 0.0;
  for(i = 0; i < nbin; i++)
    for(j = 0; j < nbin; j++) mu -= Lam[j]*W[i+j*nbin]*W[i+j*nbin];
  for(i = 0; i < nbin; i++) Lam[i] /= mu;
}
void pGTRm(double t, int nchar, double *fr, double *W, double *Lam, 
	   int deriv, double *M, double *Mp, double *Mpp){
  int i,j,k;
  double w;
  for(i = 0; i < nchar*nchar; i++) M[i]=0.0;
  if(deriv==1 || deriv==2) for(i = 0; i < nchar*nchar; i++) Mp[i]=0.0;
  if(deriv==2) for(i = 0; i < nchar*nchar; i++) Mpp[i]=0.0;
  
  for(i = 0; i < nchar; i++)
    for(j = 0; j < nchar; j++){
      for(k = 0; k < nchar; k++){
	w=W[i + k*nchar] * W[j+ k*nchar] * exp(Lam[k]*t);
	M[i+j*nchar] += w;
	if(deriv==1 || deriv==2) Mp[i+j*nchar] += Lam[k]*w;
	if(deriv==2) Mpp[i+j*nchar] += Lam[k]*Lam[k]*w;
      }
      M[i+j*nchar] /= fr[i];
      if(deriv==1 || deriv==2) Mp[i+j*nchar] /= fr[i];
      if(deriv==2) Mpp[i+j*nchar] /= fr[i];
    }
  rmnegtrans(nchar,M);
}
void pGTRma(double t, int nchar, double *fr, double *W, double *Lam, 
	    int deriv, double alpha, double *M, double *Mp, double *Mpp){
  int i,j,k;
  double dt,de[2];
  for(i = 0; i < nchar*nchar; i++) M[i]=0.0;
  if(deriv > 0) for(i = 0; i < nchar*nchar; i++) Mp[i]=0.0;
  if(deriv==2 || deriv==4) for(i = 0; i < nchar*nchar; i++) Mpp[i]=0.0;
  
  for(i = 0; i < nchar; i++)
    for(j = 0; j < nchar; j++){
      for(k = 0; k < nchar; k++){
	dt=int_elamrt(t,Lam[k],alpha,deriv,de);
	M[i+j*nchar] += W[i + k*nchar] * W[j+ k*nchar] * dt;
	if(deriv > 0){
	  Mp[i+j*nchar] += W[i + k*nchar] * W[j+ k*nchar] * de[0];
	}
	if(deriv==2 || deriv==4)
	  Mpp[i+j*nchar] += W[i + k*nchar] * W[j+ k*nchar] * de[1];
      }
      M[i+j*nchar] /= fr[i];
      if(deriv > 0) Mp[i+j*nchar] /= fr[i];
      if(deriv==2 || deriv==4) Mpp[i+j*nchar] /= fr[i];
    }
  rmnegtrans(nchar,M);
}
int subst_model_setpm(int smodel,
		      void (*(*pm))(double t, int nchar, double *fr, 
				    double *W, double *Lam, int deriv, 
				    double *M, double *Mp, 
				    double *Mpp)){
  /* IN:
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   *
   * OUT:
   * pm - transition matrix function 
   */
  switch(smodel){
  case 0:
    *pm=&pJCm_gen; 
    break;
  case 1:
    *pm=&pF81m_gen; 
    break;
  case 2:
    *pm=&pF84m_gen; ;
    break;
  case 3: case 4: case 5: case 6: case 7: case 9:
    *pm=&pGTRm; 
    break;
  case 8:
    *pm=&pHKYm_gen;
    break;
  }
  return(0);
}
int subst_model_setpma(int smodel,
		       void (*(*pm))(double t, int nchar, double *fr, 
				     double *W, double *Lam, int deriv, 
				     double alpha,
				     double *M, double *Mp, double *Mpp)){
  /* IN:
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   *
   * OUT:
   * pm - transition matrix function 
   */
  switch(smodel){
  case 0:
    *pm=&pJCma_gen; 
    break;
  case 1:
    *pm=&pF81ma_gen; 
    break;
  case 2:
    *pm=&pF84ma_gen; ;
    break;
  case 3: case 4: case 5: case 6: case 7: case 9:
    *pm=&pGTRma; 
    break;
  case 8:
    *pm=&pHKYma_gen;
    break;
  }
  return(0);
}
int subst_model_setparam(int nchar, int smodel, double *fr, double *Qk,
			 double **W, double **Lam){
  /* IN:
   * nchar - number of character states
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   * Qk - rate matrix for GTR, ttratio, K or Kappa for F84, HKY, < 0 => ttratio; 
   * not used otherwise. 
   *
   * IN/OUT:
   * fr - frequencies; reset for JC
   *      if *fr < 0 for aa models, model-based frequencies used
   * OUT:
   * W,Lam - contain parameters for substitution model
   *
   * RETURN:
   * 2,1 or 0 according to whether both W and Lam were allocated, just W or
   * neither
   */
  int allocWLam=0,aamodel=0,i;
  double *fro,*Wo,*Lamo;
  if (smodel==4 || smodel==5 || smodel==6 || smodel==7 || smodel==9) aamodel=1;

  if (smodel==0){ /* JC */
    for(i = 0; i < nchar; i++) fr[i] = 1.0/nchar;
  }
  if (smodel==1){ /* F81 */
    *W=(double *) malloc((size_t) sizeof(double));
    **W=muF81f(nchar,fr);
/*     for(i = 0; i < nchar; i++) printf("%f\n",fr[i]); printf("\n"); */
/*     printf("%f\n",**W); */
    allocWLam=1;
  }
  if (smodel==2){ /* F84 */
    *W=(double *) malloc((size_t) sizeof(double));
    *Lam=(double *) malloc((size_t) sizeof(double));
/*     printf("%.16e\n",*Qk); */
/*     for(i = 0; i < nchar; i++) printf("%f\n",fr[i]); printf("\n"); */
    **Lam=*Qk;
    if(*Qk < 0) 
      **Lam=tt2K(-(*Qk),fr);
    **W= muF84f(**Lam,fr);
/*     printf("%f %f\n",**W,**Lam); */
    allocWLam=2;
  }
  if (smodel==3){ /* GTR */
    *W = (double *) malloc((size_t) nchar*nchar*sizeof(double));
    *Lam = (double *) malloc((size_t) nchar*sizeof(double));
    Q2frWLam(nchar,Qk,fr,*W,*Lam);
    allocWLam=2;
  }
  if (smodel==4){ /* PAM */
    Wo = pamprobmat; Lamo = pameigmat;
  }
  if (smodel==5){ /* JTT */
    Wo = jttprobmat; Lamo = jtteigmat;
  }
  if (smodel==6){ /* WAG */
    Wo = wagprobmat; Lamo = wageigmat;
  }
  if (smodel==7){ /* mtREV24 */
    Wo = mtREV24probmat; Lamo = mtREV24eigmat;
  }
  if (smodel==8){ /* HKY */
    *W=(double *) malloc((size_t) sizeof(double));
    *Lam=(double *) malloc((size_t) sizeof(double));
    **Lam=*Qk;
    if(*Qk < 0) 
      **Lam=tt2kappa(-(*Qk),fr);
    **W = muHKYf(**Lam,fr);
    allocWLam=2;
  }
  if(smodel==9){ /* LG */
    Wo = lgprobmat; Lamo = lgeigmat;
  }
  if (aamodel){  /* amino acid models */
    if (*fr > 0){ /* different frequencies than for model */
      *W = (double *) malloc((size_t) nchar*nchar*sizeof(double));
      *Lam = (double *) malloc((size_t) nchar*sizeof(double));
      fro = (double *) malloc((size_t) nchar*sizeof(double));
      makeprotfreqs(fro,Wo,Lamo);
      memcpy(*W,Wo,(size_t) nchar*nchar*sizeof(double));
      memcpy(*Lam,Lamo,(size_t) nchar*sizeof(double));
      chQ_freq(nchar,*W,*Lam,fr,fro);
      allocWLam=2;
    }
    else{
      *W=Wo; *Lam=Lamo;
      makeprotfreqs(fr,*W,*Lam);
    }
  }
  return(allocWLam);
}
void transm_rate(int smodel, int nchar, int ntaxa, double *utreec, 
		 int nrate, double *rate, double alpha,
		 double *fr, double *W, double *Lam, int deriv, 
		 double *M, double *Mp, double *Mpp){
  int nchar2,nb,j,join,kup,r,i,k;
  double tr,one,*Mpt,*Mppt;
  void (*pm)(double t, int nchar, double *fr, double *W, double *Lam, 
	     int deriv, double *M, double *Mp, double *Mpp);
  void (*pma)(double t, int nchar, double *fr, double *W, double *Lam, 
	      int deriv, double alpha, double *M, double *Mp, double *Mpp);
  if (alpha < 0)
    subst_model_setpm(smodel,&pm);
  if (alpha > 0) 
    subst_model_setpma(smodel,&pma);
  nchar2 = nchar*nchar;
  nb = 2*ntaxa-3;
  for(j = 0; j < nrate; j++){
    for(join = 0; join < ntaxa-1; join++){
      kup = (join < ntaxa-2)?2:1;
      for(k = 0; k < kup; k++){
	r = (int) utreec[k+join*4];
	tr = utreec[k+2+join*4]*rate[j];
	Mpt=&one; Mppt=&one;
	if(deriv > 0) Mpt=&Mp[r*nchar2+j*nchar2*nb];
	if(deriv==2||deriv==4) Mppt=&Mpp[r*nchar2+j*nchar2*nb];
	if(alpha < 0)
	  (*pm)(tr,nchar,fr,W,Lam,deriv,&M[r*nchar2+j*nchar2*nb],Mpt,Mppt);
	if(alpha > 0)
	  (*pma)(tr,nchar,fr,W,Lam,deriv,alpha,&M[r*nchar2+j*nchar2*nb],
		 Mpt,Mppt);	
	if(deriv==1 || deriv==2)
	  for(i = 0; i < nchar2; i++) 
	    Mp[i+r*nchar2+j*nchar2*nb] *= rate[j];
	if(deriv==2)
	  for(i = 0; i < nchar2; i++) 
	    Mpp[i+r*nchar2+j*nchar2*nb] *= rate[j]*rate[j];
      }}}
}
int mldist(double *a, double *b, double *t, double *fr,
	   double *M, double *Mp, double *Mpp,
	   int nchar, int *nxy, int nrate, double *wt, double *f){
  int j,x,y,nchar2;
  double fxy[3],w;

  nchar2=nchar*nchar;
  /* printf("%f\n",*t); */
  /* for(j = 0; j < nrate; j++){ */
  /*   printf("%f\n",wt[j]); */
  /*   wmat(stdout,nchar,nchar,&M[j*nchar2]); */
  /*   printf("\n"); */
  /*   wmat(stdout,nchar,nchar,&Mp[j*nchar2]); */
  /*   printf("\n"); */
  /* } */
  /* exit(0); */
  
  for(j = 0; j < 3; j++) f[j]=0.0;
  for(x = 0; x < nchar; x++)
    for(y = 0; y < nchar; y++){
      for(j = 0; j < 3; j++) fxy[j] = 0.0;
      for(j = 0; j < nrate; j++){
	fxy[0] += wt[j]*fr[x]*M[x+y*nchar+j*nchar2];
	fxy[1] += wt[j]*fr[x]*Mp[x+y*nchar+j*nchar2];
	fxy[2] += wt[j]*fr[x]*Mpp[x+y*nchar+j*nchar2];
      }
      f[0] += nxy[x+y*nchar]*log(fxy[0]);
      w=fxy[1]/fxy[0];
      f[1] += nxy[x+y*nchar]*w;
      f[2] += nxy[x+y*nchar]*(fxy[2]/fxy[0]-w*w);
    }
  /* printf("%f %e %e\n",*t,f[0],f[1]); */
  /* exit(0); */
  return(bnewtr(a,b,t,f));
}
void complf_pair(double *fr, double *M, double *Mp, double *Mpp, 
		 int nchar, int *nxy, int deriv, double *fp){
  int i,x,y;
  double te;
  for(i = 0; i < 3; i++) fp[i]=0.0;
  for(x = 0; x < nchar; x++)
    for(y = 0; y < nchar; y++){
      fp[0] += nxy[x+y*nchar]*log(fr[x]*M[x+y*nchar]);
      if(deriv==3 || deriv==4){
	te = Mp[x+y*nchar]/M[x+y*nchar];
	fp[1] += nxy[x+y*nchar]*te;
	/* printf("%i %i %i %f %f\n",x,y,nxy[x+y*nchar],te,fp[1]); */
      }
      if(deriv==4)
	fp[2] += nxy[x+y*nchar]*(Mpp[x+y*nchar]/M[x+y*nchar] - te*te);
    }
}
void complf(int smodel, int nchar, int ntaxa, double alpha, 
	    double *fr, double *W, double *Lam, int *nxy, double *dmat,
	    int deriv, double *M, double *Mp, double *Mpp, double *f){
  int idx=0,i,p1,p2,nchar2;
  double one=1.0,utreec[3],fp[3];
  nchar2=nchar*nchar;
  for(i = 0; i < 3; i++) f[i]=0.0;
  for(p1 = 0; p1 < ntaxa; p1++)
    for(p2 = p1+1; p2 < ntaxa; p2++){
      utreec[0]=0; utreec[2]=dmat[p1 + p2*ntaxa];
      transm_rate(smodel,nchar,2,utreec,1,&one,alpha,fr,W,Lam,deriv,
		  &M[idx*nchar2],&Mp[idx*nchar2],&Mpp[idx*nchar2]);
/*       wmat(stdout,nchar,nchar,&M[idx*nchar2]); */
/*       wmat(stdout,nchar,nchar,&Mp[idx*nchar2]); */
/*       wmat(stdout,nchar,nchar,&Mpp[idx*nchar2]); */
      complf_pair(fr,&M[idx*nchar2],&Mp[idx*nchar2],&Mpp[idx*nchar2],
		  nchar,&nxy[idx*nchar2],deriv,fp);
/*       printf("%i %f %e %e\n",idx,fp[0],fp[1],fp[2]); */
/* 	exit(0); */
      for(i = 0; i < 3; i++) f[i] += fp[i];
      idx++;
    }
}
double mladistf_pair(int smodel, int nchar, int *nxy, 
		   int nrate, double *rate, double *wt, double alpha, 
		   double *fr, double *W, double *Lam, 
		   double *M, double *Mp, double *Mpp, double *d){
  double a,b,utreec[3],f[3];
  a=1.0e-6; b=100.0;
  do{
    utreec[0]=0; utreec[2]=*d;
    transm_rate(smodel,nchar,2,utreec,nrate,rate,alpha,fr,W,Lam,2,M,Mp,Mpp);
  }while(mldist(&a,&b,d,fr,M,Mp,Mpp,nchar,nxy,nrate,wt,f));
  /* exit(0); */
  return(f[0]);
}
void mladistf(int smodel, int nchar, int nsite, int ntaxa, int *seq, 
		  int nrate, double *rate, double *wt, 
		  int fix_alpha, double *alpha, 
		  double *fr, double *W, double *Lam, 
		  int *nv, int *nxy, double *M, double *Mp, double *Mpp, 
		  double *dmat){
  int p1,p2,iter,itmax=100,idx,nchar2,i;
  double f0[3],f[3],tol=1.0e-6,fopt=-1.0e+20,aa,ba;

  char (*name)[11];
  name = (char (*)[11]) malloc((size_t) ntaxa*sizeof(*name));
  for(i = 0; i < ntaxa; i++){sprintf(name[i],"%i",i);}
  for(p1 = 0; p1 < ntaxa*ntaxa; p1++) dmat[p1]=0.1;
  for(p1 = 0; p1 < ntaxa; p1++) dmat[p1+p1*ntaxa]=0.0;
  
  nchar2=nchar*nchar;
  if(!fix_alpha) *alpha=1;

  for(idx = 0, p1 = 0; p1 < ntaxa; p1++)
    for(p2 = p1+1; p2 < ntaxa; p2++, idx++)
      nv[idx]=pchar_freqi(nchar,nsite,&seq[p1*nsite],&seq[p2*nsite],
			  &nxy[idx*nchar2]);
  
  for(iter = 1; iter <= itmax; iter++){
    for(idx=0, p1 = 0; p1 < ntaxa; p1++)
      for(p2 = p1+1; p2 < ntaxa; p2++, idx++){
	mladistf_pair(smodel,nchar,&nxy[idx*nchar2],
		      nrate,rate,wt,*alpha,
		      fr,W,Lam,
		      &M[idx*nchar2*nrate],
		      &Mp[idx*nchar2*nrate],
		      &Mpp[idx*nchar2*nrate],&dmat[p1 + p2*ntaxa]);
/* 	jcdist(nchar,&nxy[idx*nchar2],*alpha); */
	dmat[p2 + p1*ntaxa] = dmat[p1 + p2*ntaxa];
      }
/*     pr_distmat(stdout,ntaxa,name,dmat); */
    if(fix_alpha) break;

    complf(smodel,nchar,ntaxa,-1.0,fr,W,Lam,nxy,dmat,0,M,Mp,Mpp,f0);
    aa=1.0e-6; ba=100.0; 
    if(*alpha<aa || *alpha>ba) *alpha=1.0;
    do{
      complf(smodel,nchar,ntaxa,*alpha,fr,W,Lam,nxy,dmat,4,M,Mp,Mpp,f);
      /* 	printf("%f %f %f %f\n",*alpha,f[0],f[1],f[2]); */
    }while(bnewtr(&aa,&ba,alpha,f));
    if(f0[0] > f[0]){
      *alpha=-1.0;
      f[0]=f0[0];
    }
/*     printf("%i %f %f\n",iter,*alpha,f[0]); */
    
    if(f[0]-fopt < tol) break;
    fopt=f[0];
  }
  free(name);
}
void mladistf_paramf(FILE *fp, int *nchar, int *model, char *aaRmat, 
		     FILE **Qfile, double *kappa, double *alpha, int *ncatG, 
		     int *nrate, FILE **ratefile, int *fix_alpha, 
		     FILE **seqfile){
  char c,opt[100],value[100];
  int is_option,done=0;
  *fix_alpha=1;
  *Qfile=NULL;
  *ratefile=NULL;
  *seqfile=NULL;
  *aaRmat='\0';
  *nchar=4;
  *ncatG=0;
  *alpha=-1;
  *nrate=1;
  while(!done){
    c = ctl_paraml(fp,opt,value,&is_option);
    if(is_option){
/*       printf("option: %s value: %s\n",opt, value); */
      if(strcmp(opt,"nchar")==0) *nchar=atoi(value);
      if(strcmp(opt,"model")==0) *model=atoi(value);
      if(strcmp(opt,"aaRatefile")==0) sprintf(aaRmat,"%s",value);
      if(strcmp(opt,"Qfile")==0) 
	*Qfile=fopene(value,"r","Qfile not available");
      if(strcmp(opt,"kappa")==0) *kappa=atof(value);
      if(strcmp(opt,"ttratio")==0) *kappa=-atof(value);
      if(strcmp(opt,"fix_alpha")==0) *fix_alpha=atoi(value);
      if(strcmp(opt,"alpha")==0) *alpha=atof(value);
      if(strcmp(opt,"ncatG")==0) *ncatG=atoi(value);
      if(strcmp(opt,"nrate")==0) *nrate=atoi(value);
      if(strcmp(opt,"ratefile")==0)
	*ratefile=fopene(value,"r","ratefile not available");
      if(strcmp(opt,"seqfile")==0) 
	*seqfile=fopene(value,"r","seqfile not available");
    }
    if(c==EOF) done=1;
  }
  return;
}
void mladistf_getdp(FILE *infile, FILE *Qfile, FILE *ratefile, int model, 
		   int nchar, int numsp, int nsite, int ncatG, double kappa, 
		   double *alpha, char *aaRmat, int *nrates, int *smodel, 
		   char name[][11], int *seq, double *fr, double **W, 
		   double **Lam, double *rate, double *prob){
  char *seqc;
  int i,j;
  double *Qk;

  *smodel=pmodel2smodel(model,aaRmat,nchar);  

  seqc = (char *) malloc((size_t) numsp*nsite*sizeof(char));
  if(*smodel==3) 
    Qk = (double *) malloc((size_t) nchar*nchar*sizeof(double));

  /* read in sequence data */
  rinterleavef_nolim(infile, &numsp, &nsite, name, seqc);
  letter(numsp, nsite, seqc);
  if(nchar==4) 
    for(i = 0; i < nsite*numsp; i++)
      seq[i] = l2i(seqc[i]);
  if(nchar==20)
    for(i = 0; i < nsite*numsp; i++)
      seq[i] = l2ip(seqc[i]);
  if(aaRmat!='\0' && model==2){
    *fr=-1; /* database frequencies for amino acid models */
  }
  else if(*smodel==0){
    for(i = 0; i < nchar; i++) fr[i] = 1.0/nchar;
  }
  else{
    char_freq(nchar, nsite, numsp, seq, fr);
  }

  /* K for F84, kappa for HKY */
  if(*smodel==2 || *smodel==8) Qk=&kappa;
  /* Q matrix for GTR */
  if (*smodel == 3)
    for(i = 0; i < nchar; i++) 
      for(j = 0; j < nchar; j++) fscanf(Qfile,"%lf",&Qk[i+j*nchar]);
  subst_model_setparam(nchar,*smodel,fr,Qk,W,Lam);

  
  /* rate distribution */
  if(*alpha > 0 && ncatG > 0){
    DiscreteGamma(prob,rate,*alpha,*alpha,*nrates,0);
    /* for(i = 0; i < *nrates; i++) printf("%f %f\n", rate[i], prob[i]); */
    *alpha=-1;
  }
  if(*alpha < 0 && ratefile!=NULL){ 
    for(i = 0; i < *nrates; i++) fscanf(ratefile, "%lf %lf", &rate[i], &prob[i]);
/*     for(i = 0; i < *nrates; i++) printf("%f %f\n", rate[i], prob[i]); */
    *alpha=-1;
  }
  if(ratefile==NULL && ncatG==0){
    *nrates=1; *rate=1.0; *prob=1.0;
  }

  free(seqc);
  if (*smodel == 3) free(Qk);
}
double *cvec(int n)
{
  double *x;

  x=(double *) malloc((size_t) n*sizeof(double));

  return(x);
}
int icomp(const int *a, const int *b) 
{
  return((*a <= *b)?-1:1);
}
void isort(int n, int *x)
{
  qsort(x, n, (size_t) sizeof(int), (int (*)(const void *, const void *)) icomp);
}
double urand(void){
  return(drand48());  
/*   return(((double) rand())/(RAND_MAX+1.0)); /\* windows *\/ */
}
double ipow(double x, int n) /*x**n, n>=0*/
{
  int i;
  double f=1.0;

  for(i=1; i<=n; i++) f *= x;
  return(f);
}
void rnorm(int n, double *v, double *se)    /* Generate n N(0,1)*/
{
  int i;
  double x1, x2;

  for(i=0; i<(n-1); i+=2)
    {x1 = urand(); x2 = urand(); 
    v[i] = sqrt(-2.0*log(x1))*sin(2.0*PI*x2);
    v[i+1] = sqrt(-2.0*log(x1))*cos(2.0*PI*x2);}

  if (n%2 != 0)
    {x1 = urand(); x2 = urand();
    v[n-1] = sqrt(-2.0*log(x1))*sin(2.0*PI*x2);}
}
double pnorm(double x, double a, double b)
{
  /* IN:
   * x - the value for with P(X <= x) is required
   * a - the mean of the normal distribution of X
   * b - the standard deviation of the normal distribution
   *
   * RETURN:
   * P(X <= x) */
  double rt2 = 1.41421356237309515, xhi = 2.66e+1;

  double z, xt,t, y;
  
  z = (x - a)/(rt2 * b);
  xt = fabs(z);
  if (xt >= xhi){
    return((z < 0.0)?0.0:1.0);
  }
  t = 1.0 - 7.5/(xt + 3.75);
  y = (((((((((((((((+3.328130055126039e-10
		     *t-5.718639670776992e-10)*t-4.066088879757269e-9)
		   *t+7.532536116142436e-9)*t+3.026547320064576e-8)
		 *t-7.043998994397452e-8)*t-1.822565715362025e-7)
	       *t+6.575825478226343e-7)*t+7.478317101785790e-7)
	     *t-6.182369348098529e-6)*t+3.584014089915968e-6)
	   *t+4.789838226695987e-5)*t-1.524627476123466e-4)
	 *t-2.553523453642242e-5)*t+1.802962431316418e-3)
       *t-8.220621168415435e-3)*t+2.414322397093253e-2;
  y = (((((y*t-5.480232669380236e-2)*t+1.026043120322792e-1)
	 *t-1.635718955239687e-1)*t+2.260080669166197e-1)
       *t-2.734219314954260e-1)*t + 1.455897212750385e-1;
  return((z > 0.0)?1.0 - 0.5*exp(-xt*xt)*y:0.5*exp(-xt*xt)*y);
}
double pchisq(double x, int df)
{
  /* IN:
   * x - the value for with P(X <= x) is required
   * df - X above is a chi-squared random variable w/ df degrees of freedom 
   *
   * RETURN:
   * P(X <= x) */
  int i;
  double f, dchi;

  if (df%2 == 0){
    dchi = 0.50*exp(-0.50*x);
    f = 1.0 - 2.0*dchi;
    for(i = 4; i <= df; i+=2){
      dchi *= x/((double) i - 2);
      f -= 2.0*dchi;
    }
  }
  else{
    f = 2.0*pnorm(sqrt(x),0.0,1.0)-1.0;
    dchi = exp(-0.5*x)/sqrt(2.0*PI*x);
    for(i = 3; i <= df; i+=2){
      dchi *= x/((double) i - 2);
      f -= 2.0*dchi;
    }
  }

  return(f);
}
int lstring(char * string){
  /* returns length of string */
  int l = 0;
  while (string[l] != '\0') l++;
  return(l);
}
void dorder(int n, double *x, int *o){
  int i,j,to,io;
  double xm; 

  for(i = 0; i < n; i++) o[i] = i;
  for(i = 0; i < n; i++){
    xm = x[o[i]];
    to = o[i];
    io = i;
    for(j = i+1; j < n; j++)
      if (x[o[j]] < xm){
	to = o[j];
	xm = x[o[j]];
	io = j;
      }
    o[io] = o[i];
    o[i] = to;
  }
}
void iorder(int n, int *x, int *o){
  int i,j,to,io,xm; 

  for(i = 0; i < n; i++) o[i] = i;
  for(i = 0; i < n; i++){
    xm = x[o[i]];
    to = o[i];
    io = i;
    for(j = i+1; j < n; j++)
      if (x[o[j]] < xm){
	to = o[j];
	xm = x[o[j]];
	io = j;
      }
    o[io] = o[i];
    o[i] = to;
  }
}
int DiscreteGamma (double freqK[], double rK[], double alpha, double beta, int K, int dgammamean)
{
/* discretization of gamma distribution with equal proportions in each 
   category.
*/
   int i;
   double t, factor=alpha/beta*K, lnga1;

   if (dgammamean==0) {
      lnga1=LnGamma(alpha+1);
      for (i=0; i<K-1; i++) /* cutting points, Eq. 9 */
         freqK[i]=InverseCDFGamma((i+1.0)/K, alpha, beta);
      for (i=0; i<K-1; i++) /* Eq. 10 */
         freqK[i]=IncompleteGamma(freqK[i]*beta, alpha+1, lnga1);

      rK[0] = freqK[0]*factor;
      rK[K-1] = (1-freqK[K-2])*factor;
      for (i=1; i<K-1; i++)  rK[i] = (freqK[i]-freqK[i-1])*factor;
   }
   else {
      for(i=0; i<K; i++) rK[i]=InverseCDFGamma((i*2.+1)/(2.*K), alpha, beta);
      for(i=0,t=0; i<K; i++) t+=rK[i];
      for(i=0; i<K; i++) rK[i]*=factor/t;
   }
   for (i=0; i<K; i++) freqK[i]=1.0/K;

   return (0);
}
void error2 (char * message)
{ printf("\nError: %s.\n", message); exit(-1); }
long factorial (int n)
{
   long f=1, i;
   if (n>10) error2("n>10 in factorial");
   for (i=2; i<=(long)n; i++) f *= i;
   return (f);
}
double LnGamma (double x)
{
/* returns ln(gamma(x)) for x>0, accurate to 10 decimal places.
   Stirling's formula is used for the central polynomial part of the procedure.

   Pike MC & Hill ID (1966) Algorithm 291: Logarithm of the gamma function.
   Communications of the Association for Computing Machinery, 9:684
*/
   double f=0, fneg=0, z, lng;
   int nx=(int)x;

   if((double)nx==x && nx>1 && nx<12)
      lng=log((double)factorial(nx-1));
   else {
      if(x<=0) {
         error2("LnGamma not implemented for x<0");
         if((int)x-x==0) { puts("lnGamma undefined"); return(-1); }
         for (fneg=1; x<0; x++) fneg/=x;
         if(fneg<0) error2("strange!! check lngamma");
         fneg=log(fneg);
      }
      if (x<7) {
         f=1;  z=x-1;
         while (++z<7)  f*=z;
         x=z;   f=-log(f);
      }
      z = 1/(x*x);
      lng = fneg+ f + (x-0.5)*log(x) - x + .918938533204673 
             + (((-.000595238095238*z+.000793650793651)*z-.002777777777778)*z
                  +.083333333333333)/x;
   }
   return  lng;
}
double InverseCDFNormal (double prob)
{
/* returns z so that Prob{x<z}=prob where x ~ N(0,1) and (1e-12)<prob<1-(1e-12)
   returns (-9999) if in error
   Odeh RE & Evans JO (1974) The percentage points of the normal distribution.
   Applied Statistics 22: 96-97 (AS70)

   Newer methods:
     Wichura MJ (1988) Algorithm AS 241: the percentage points of the
       normal distribution.  37: 477-484.
     Beasley JD & Springer SG  (1977).  Algorithm AS 111: the percentage 
       points of the normal distribution.  26: 118-121.
*/
   double a0=-.322232431088, a1=-1, a2=-.342242088547, a3=-.0204231210245;
   double a4=-.453642210148e-4, b0=.0993484626060, b1=.588581570495;
   double b2=.531103462366, b3=.103537752850, b4=.0038560700634;
   double y, z=0, p=prob, p1;

   p1 = (p<0.5 ? p : 1-p);
   if (p1<1e-20) z=999;
   else {
      y = sqrt (log(1/(p1*p1)));   
      z = y + ((((y*a4+a3)*y+a2)*y+a1)*y+a0) / ((((y*b4+b3)*y+b2)*y+b1)*y+b0);
   }
   return (p<0.5 ? -z : z);
}
double InverseCDFChi2 (double prob, double v)
{
/* returns z so that Prob{x<z}=prob where x is Chi2 distributed with df=v
   returns -1 if in error.   0.000002<prob<0.999998
   RATNEST FORTRAN by
       Best DJ & Roberts DE (1975) The percentage points of the 
       Chi2 distribution.  Applied Statistics 24: 385-388.  (AS91)
   Converted into C by Ziheng Yang, Oct. 1993.
*/
   double e=.5e-6, aa=.6931471805, p=prob, g, small=1e-6;
   double xx, c, ch, a=0,q=0,p1=0,p2=0,t=0,x=0,b=0,s1,s2,s3,s4,s5,s6;

   if (p<small)   return(0);
   if (p>1-small) return(9999);
   if (v<=0)      return (-1);

   g = LnGamma (v/2);
   xx=v/2;   c=xx-1;
   if (v >= -1.24*log(p)) goto l1;

   ch=pow((p*xx*exp(g+xx*aa)), 1/xx);
   if (ch-e<0) return (ch);
   goto l4;
l1:
   if (v>.32) goto l3;
   ch=0.4;   a=log(1-p);
l2:
   q=ch;  p1=1+ch*(4.67+ch);  p2=ch*(6.73+ch*(6.66+ch));
   t=-0.5+(4.67+2*ch)/p1 - (6.73+ch*(13.32+3*ch))/p2;
   ch-=(1-exp(a+g+.5*ch+c*aa)*p2/p1)/t;
   if (fabs(q/ch-1)-.01 <= 0) goto l4;
   else                       goto l2;
  
l3: 
   x = InverseCDFNormal(p);
   p1 = 0.222222/v;
   ch = v*pow((x*sqrt(p1)+1-p1), 3.0);
   if (ch>2.2*v+6)
      ch = -2*(log(1-p)-c*log(.5*ch)+g);
l4:
   q=ch;   p1=.5*ch;
   if ((t=IncompleteGamma (p1, xx, g))<0)
      error2 ("\nIncompleteGamma");
   p2=p-t;
   t=p2*exp(xx*aa+g+p1-c*log(ch));   
   b=t/ch;  a=0.5*t-b*c;

   s1=(210+a*(140+a*(105+a*(84+a*(70+60*a))))) / 420;
   s2=(420+a*(735+a*(966+a*(1141+1278*a))))/2520;
   s3=(210+a*(462+a*(707+932*a)))/2520;
   s4=(252+a*(672+1182*a)+c*(294+a*(889+1740*a)))/5040;
   s5=(84+264*a+c*(175+606*a))/2520;
   s6=(120+c*(346+127*c))/5040;
   ch+=t*(1+0.5*t*s1-b*c*(s1-b*(s2-b*(s3-b*(s4-b*(s5-b*s6))))));
   if (fabs(q/ch-1) > e) goto l4;

   return (ch);
}
double IncompleteGamma (double x, double alpha, double ln_gamma_alpha)
{
/* returns the incomplete gamma ratio I(x,alpha) where x is the upper 
           limit of the integration and alpha is the shape parameter.
   returns (-1) if in error
   ln_gamma_alpha = ln(Gamma(alpha)), is almost redundant.
   (1) series expansion,     if (alpha>x || x<=1)
   (2) continued fraction,   otherwise
   RATNEST FORTRAN by
   Bhattacharjee GP (1970) The incomplete gamma integral.  Applied Statistics,
   19: 285-287 (AS32)
*/
   int i;
   double p=alpha, g=ln_gamma_alpha;
   double accurate=1e-10, overflow=1e60;
   double factor, gin=0, rn=0, a=0,b=0,an=0,dif=0, term=0, pn[6];

   if (x==0) return (0);
   if (x<0 || p<=0) return (-1);

   factor=exp(p*log(x)-x-g);   
   if (x>1 && x>=p) goto l30;
   /* (1) series expansion */
   gin=1;  term=1;  rn=p;
 l20:
   rn++;
   term *= x/rn;   gin += term;
   if (term > accurate) goto l20;
   gin *= factor/p;
   goto l50;
 l30:
   /* (2) continued fraction */
   a = 1-p;   b = a+x+1;  term = 0;
   pn[0] = 1;  pn[1] = x;  pn[2] = x+1;  pn[3] = x*b;
   gin = pn[2]/pn[3];
 l32:
   a++;  
   b += 2;
   term++;
   an = a*term;
   for (i=0; i<2; i++) 
      pn[i+4] = b*pn[i+2] - an*pn[i];
   if (pn[5] == 0) goto l35;
   rn = pn[4]/pn[5];
   dif = fabs(gin-rn);
   if (dif > accurate) goto l34;
   if (dif <= accurate*rn) goto l42;
 l34:
   gin = rn;
 l35:
   for (i=0; i<4; i++) pn[i] = pn[i+2];
   if (fabs(pn[4]) < overflow) goto l32;
   for (i=0; i<4; i++) pn[i] /= overflow;
   goto l32;
 l42:
   gin = 1-factor*gin;

 l50:
   return (gin);
}
int sqrtmat(int m, double *A, double *Ah){
  /* returns 0 if eigenvalue had to be set to 0, 1 o/w */
  int i,j,k,izero=0;
  double *R,*Lam,*Offdiag,eps=0.01;
  
  R = (double  *) malloc((size_t) m*m*sizeof(double));
  Offdiag = (double  *) malloc((size_t) m*m*sizeof(double));
  Lam = (double  *) malloc((size_t) m*sizeof(double));

  memcpy(R,A,m*m*sizeof(double));
  eigenRealSym(R,m,Lam,Offdiag);
  for(k = 0; k < m; k++) 
    if (Lam[k] < 0 && Lam[k] > -eps){
      Lam[k]=0.0;
      izero=1;
    }
  for(i = 0; i < m*m; i++) Ah[i] = 0.0;
   for(i = 0; i < m; i++)
    for(j = 0; j < m; j++)
      for(k = 0; k < m; k++) 
	Ah[i+j*m] += R[k+i*m]*sqrt(Lam[k])*R[k+j*m];
   
   free(R); free(Lam); free(Offdiag);
   return(izero);
}
void rmvnorm(int nsim, int m, double *mu, double *Sigma, double *V){
  int i,j,isim;
  double half=0.5,*Z,*Shalf;

  Z = (double  *) malloc((size_t) m*sizeof(double));
  Shalf = (double  *) malloc((size_t) m*m*sizeof(double));
  
  sqrtmat(m,Sigma,Shalf);
  /* wmat(stdout,3,3,Sigma); */
  /* wmat(stdout,3,3,Shalf); */

  for(isim = 0; isim < nsim; isim++){
    rnorm(m,Z,&half);
    for(i = 0; i < m; i++){
      V[i+isim*m] = 0.0;
      for(j = 0; j < m; j++)
      	V[i+isim*m] += Shalf[i+j*m]*Z[j];
      V[i+isim*m] += mu[i];
    }
  }

  free(Z); free(Shalf); 
}
int read_tree(FILE *fp, char *tstring)
{
  /* assumes 
   *   '[' and ']' delimit comments only
   *    '][' won't be used to end a comment and start a new one
   *   white space can be removed */
  int ltstring;
  char c;

  ltstring = 0;
  while((c = getc(fp)) != ';'){ 
    if (c != ' ' && c != '\t' && c != '\r' && c != '\n'){ /* no white space */
      if (c == '[') /* start of comment; ignored */
	while((c = getc(fp)) != ']') ;
      else
	tstring[ltstring++]=c;
    }
  } 
  tstring[ltstring++]=';';
  tstring[ltstring++]='\0';
  return(ltstring);
}
double get_dlist(char *tstr, int ss, int de, int *ssn, int *ls, int *le)
{
  /* IN:
   * tstr - tree
   * ss - char position to start search
   * de - end position of dlist to be searched
   * OUT:
   * ssn - char position to start new search
   * ls - start pos of label; (ls-1) gives end of dlist found
   * le - end pos of label; 
   * RETURN: edge length (0.1 if not present) */

  char c,els[20]; /* note the limit on edge-length string */
  int lb=0,rb=0,lrb,i,j=0;
  double el=0.1;

  i=ss;
  lrb=ss-1;
  while(i <= de-1){
    if ((c=tstr[i]) == '(') lb++;
    if (c == ')'){rb++; lrb=i;}
    if (c == ',' && lb==rb) break;
    i++;
  }
  *ssn=i+1;
  *ls=lrb+1;
  i=lrb+1; 
  while((c=tstr[i]) != ':' && c != ','&& c != ')') i++;
  *le=i-1;
  if(c == ':'){
    while((c=tstr[++i]) != ',' && c != ')') els[j++]=c;
    els[j]='\0';
    el=atof(els);
  }
  return(el);
}
void underscore2blank(char *s){
  while(*s++ != '\0')
    if(*s=='_') *s=' ';
}
char **treecs(int ntaxa, char *tstr, char names[][11], double *utreec, 
	      int has_names){
  /* IN:
   * tstr - Newick tree w/o comments, whitespace and ; (see read_tree)
   * has_names = 1 if names are available
   * IN/OUT:
   * names - if has_names, used to determine ordering 0,...,ntaxa-1 of 
   *         terminal labels
   *         o/w gives names of the taxa labeled 0,...,ntaxa-1 in utreec
   * OUT:
   * utreec 
   * RETURN:
   * ilabels - internal nodel labels; (ntaxa-2); might be uninitialized
   *    not allocated as a contiguous block
   *
   * COMMENTS:
   * Based on observation that a root subtree is either
   *    (a) a leaf and has no `(' or ')' 
   * or (b) a dlist + label and el, and has equal numbers of `(' and ')' 
   *
   * violations of Newick standard
   * - quoted labels not allowed
   * - '[' and ']' only used to delimit comments
   * - limits of 10 non-null chars on leaf names
   * - underscores are not converted to blanks
   */

  int *ds; /* starting char positions of dlists */
  int *de; /* ending char positions of dlists */
  int cd=0,td=1; /* current and total dlists */
  int cj; /* current join */
  int cil; /* current internal label */
  int ctl=0; /* current terminal label */
  int *ul; /* utreec labels */
  int *ls,*le,*is_leaf,k,i,ss,K,j,ssn,ir,il;
  char **ilabels,tname[11];
  double *el; 

  el = (double *) malloc((size_t) ntaxa*sizeof(double));
  ds = (int *) malloc((size_t) 2*ntaxa*sizeof(int));
  de = (int *) malloc((size_t) 2*ntaxa*sizeof(int));
  ls = (int *) malloc((size_t) ntaxa*sizeof(int));
  le = (int *) malloc((size_t) ntaxa*sizeof(int));
  is_leaf = (int *) malloc((size_t) ntaxa*sizeof(int));
  ul = (int *) malloc((size_t) ntaxa*sizeof(int));
  ilabels = (char **)malloc((size_t) (ntaxa-2)*sizeof(char *));

  /* if(has_names) */
  /*   for(i = 0; i < ntaxa; i++) underscore2blank(names[i]); */

  for(i = 0; i < ntaxa; i++) el[i]=0;
  
  *ds=0; *de=0; *is_leaf=0;
  while(tstr[*de] != '\0')(*de)++;
  while(tstr[*de] != ')') (*de)--; /* root label and edge ignored */

  cj=ntaxa-2;
  cil=2*ntaxa-3;
  while (cj >= 0){/* Main loop */

    /* obtain all root subtrees of the current dlist and update dlists */
    ss=ds[cd]+1;
    K=0;
    while(ss <= de[cd]-1){
      el[K]=get_dlist(tstr,ss,de[cd],&ssn,&ls[K],&le[K]);
      is_leaf[K]=(ls[K] == ss)?1:0;
      if (!is_leaf[K]){
	ds[td]=ss;
	de[td]=ls[K]-1;
	td++;
      }
      ss=ssn;
      K++;
    }
/*     printf("current dlist\n"); */
/*     for(j = ds[cd]; j <= de[cd]; j++) putchar(tstr[j]); */
/*     printf("\nnumber of dlists from current: %i\n",td-cd-1); */
/*     for(i = cd+1; i < td; i++){ */
/*       for(j = ds[i]; j <= de[i]; j++) putchar(tstr[j]); */
/*       putchar('\n'); */
/*     } */
/*     printf("number of subtrees: %i\n",K); */
/*     printf("cil: %i cj+ntaxa: %i\n",cil, cj+ntaxa); */

    /* attach ulabels to subtrees */
    for(i = 0; i < K-2; i++){
      ilabels[cil-ntaxa]=(char *)malloc(sizeof(char));
      ilabels[cil-ntaxa][0]='\0';
      cil--;
    }
    for(i=0; i < K; i++){
      if (!is_leaf[i]){
	ilabels[cil-ntaxa]=(char *)malloc((le[i]-ls[i]+2)*sizeof(char));
	memcpy(ilabels[cil-ntaxa],&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	ilabels[cil-ntaxa][le[i]+1-ls[i]]='\0';
	/* underscore2blank(ilabels[cil-ntaxa]); */
	ul[i]=cil;
	cil--;
      }
      if (is_leaf[i] && !has_names){
	memcpy(names[ctl],&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	for(k=le[i]+1-ls[i]; k < 10; k++) names[ctl][k]=' ';
	names[ctl][10]='\0';
	/* underscore2blank(names[ctl]); */
	ul[i]=ctl;
	ctl++;
      }
      if (is_leaf[i] && has_names){
	memcpy(tname,&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	for(k=le[i]+1-ls[i]; k < 10; k++) tname[k]=' ';
	tname[10]='\0';
	/* underscore2blank(tname); */
	for(j = 0; j < ntaxa; j++) 
	  if (strncmp(tname,names[j],11)==0) break;
	ul[i]=j;
      } 
    }/* end of attach ulabels ... */

    /* add root subtrees to utreec */
    for(i = 0; i < K-2; i++){
      utreec[cj*4]=ul[i]; utreec[1+cj*4]=cj-1+ntaxa; 
      utreec[2+cj*4]=el[i]; utreec[3+cj*4]=0.0;
      cj--;
    }
    if (ul[K-2] < ul[K-1]){
      ir=K-2;il=K-1;
    }
    else{
      ir=K-1;il=K-2;
    }
    utreec[cj*4]=ul[ir]; utreec[2+cj*4]=el[ir];
    utreec[1+cj*4]=ul[il]; utreec[3+cj*4]=el[il];
    cj--;
    cd++;

/*     pr_utreec(stdout,ntaxa-(cj+1),&utreec[(cj+1)*4]); */

  } /* end of Main loop */
/*   pr_utreec(stdout,ntaxa,utreec); */
/*   for(i = 0; i < ntaxa; i++) */
/*     printf("%s\n",names[i]); */
  
  free(is_leaf); 
  free(ul); 
  free(le); 
  free(ls); 
  free(de); 
  free(ds); 
  free(el); 
  return(ilabels);
}
void treecsnl(int ntaxa, char *tstr, char names[][11], double *utreec, 
	      int has_names){
  char **ilabels; 
  int j;
  ilabels=treecs(ntaxa, tstr, names, utreec, has_names);
  for(j=0; j<ntaxa-2; j++) free(ilabels[j]);
  free(ilabels);
}
void read_treecsnl(FILE *treefile, int ntaxa, char names[][11], double *utreec, 
		   int has_names){
  char tstring[10000];
  read_tree(treefile,tstring);
  /* printf("%s\n",tstring); */
  treecsnl(ntaxa,tstring,names,utreec,has_names);
}
int eigenRealSym(double A[], int n, double Root[], double Offdiag[])
{
/* This finds the eigen solution of a real symmetrical matrix A[n*n].  In return, 
   A has the right vectors and Root has the eigenvalues. work[n] is the working space.
   The matrix is first reduced to a tridiagonal matrix using HouseholderRealSym(), 
   and then using the QL algorithm with implicit shifts.  

   Adapted from routine tqli in Numerical Recipes in C, with reference to LAPACK
   Ziheng Yang, 23 May 2001
*/
   int status=0;
   HouseholderRealSym(A, n, Root, Offdiag);
   status=EigenTridagQLImplicit(Root, Offdiag, n, A);
   EigenSort(Root, A, n);

   return(status);
}
void EigenSort(double d[], double U[], int n)
{
/* this sorts the eigen values d[] and rearrange the (right) eigen vectors U[]
*/
   int k,j,i;
   double p;

   for (i=0;i<n-1;i++) {
      p=d[k=i];
      for (j=i+1;j<n;j++)
         if (d[j] >= p) p=d[k=j];
      if (k != i) {
         d[k]=d[i];
         d[i]=p;
         for (j=0;j<n;j++) {
            p=U[j*n+i];
            U[j*n+i]=U[j*n+k];
            U[j*n+k]=p;
         }
      }
   }
}
void HouseholderRealSym(double a[], int n, double d[], double e[])
{
/* This uses HouseholderRealSym transformation to reduce a real symmetrical matrix 
   a[n*n] into a tridiagonal matrix represented by d and e.
   d[] is the diagonal (eigends), and e[] the off-diagonal.
*/
   int m,k,j,i;
   double scale,hh,h,g,f;

   for (i=n-1;i>=1;i--) {
      m=i-1;
      h=scale=0;
      if (m > 0) {
         for (k=0;k<=m;k++)
            scale += fabs(a[i*n+k]);
         if (scale == 0)
            e[i]=a[i*n+m];
         else {
            for (k=0;k<=m;k++) {
               a[i*n+k] /= scale;
               h += a[i*n+k]*a[i*n+k];
            }
            f=a[i*n+m];
            g=(f >= 0 ? -sqrt(h) : sqrt(h));
            e[i]=scale*g;
            h -= f*g;
            a[i*n+m]=f-g;
            f=0;
            for (j=0;j<=m;j++) {
               a[j*n+i]=a[i*n+j]/h;
               g=0;
               for (k=0;k<=j;k++)
                  g += a[j*n+k]*a[i*n+k];
               for (k=j+1;k<=m;k++)
                  g += a[k*n+j]*a[i*n+k];
               e[j]=g/h;
               f += e[j]*a[i*n+j];
            }
            hh=f/(h*2);
            for (j=0;j<=m;j++) {
               f=a[i*n+j];
               e[j]=g=e[j]-hh*f;
               for (k=0;k<=j;k++)
                  a[j*n+k] -= (f*e[k]+g*a[i*n+k]);
            }
         }
      } 
      else
         e[i]=a[i*n+m];
      d[i]=h;
   }
   d[0]=e[0]=0;

   /* Get eigenvectors */
   for (i=0;i<n;i++) {
      m=i-1;
      if (d[i]) {
         for (j=0;j<=m;j++) {
            g=0;
            for (k=0;k<=m;k++)
               g += a[i*n+k]*a[k*n+j];
            for (k=0;k<=m;k++)
               a[k*n+j] -= g*a[k*n+i];
         }
      }
      d[i]=a[i*n+i];
      a[i*n+i]=1;
      for (j=0;j<=m;j++) a[j*n+i]=a[i*n+j]=0;
   }
}
int EigenTridagQLImplicit(double d[], double e[], int n, double z[])
{
/* This finds the eigen solution of a tridiagonal matrix represented by d and e.  
   d[] is the diagonal (eigenvalues), e[] is the off-diagonal
   z[n*n]: as input should have the identity matrix to get the eigen solution of the 
   tridiagonal matrix, or the output from HouseholderRealSym() to get the 
   eigen solution to the original real symmetric matrix.
   z[n*n]: has the orthogonal matrix as output

   Adapted from routine tqli in Numerical Recipes in C, with reference to
   LAPACK fortran code.
   Ziheng Yang, May 2001
*/
   int m,j,iter,niter=30, status=0, i,k;
   double s,r,p,g,f,dd,c,b, aa,bb;

   for (i=1;i<n;i++) e[i-1]=e[i];  e[n-1]=0;
   for (j=0;j<n;j++) {
      iter=0;
      do {
         for (m=j;m<n-1;m++) {
            dd=fabs(d[m])+fabs(d[m+1]);
            if (fabs(e[m])+dd == dd) break;  /* ??? */
         }
         if (m != j) {
            if (iter++ == niter) {
               status=-1;
               break;
            }
            g=(d[j+1]-d[j])/(2*e[j]);

            /* r=pythag(g,1); */

            if((aa=fabs(g))>1)  r=aa*sqrt(1+1/(g*g));
            else                r=sqrt(1+g*g);

            g=d[m]-d[j]+e[j]/(g+SIGN(r,g));
            s=c=1;
            p=0;
            for (i=m-1;i>=j;i--) {
               f=s*e[i];
               b=c*e[i];

               /*  r=pythag(f,g);  */
               aa=fabs(f); bb=fabs(g);
               if(aa>bb)       { bb/=aa;  r=aa*sqrt(1+bb*bb); }
               else if(bb==0)             r=0;
               else            { aa/=bb;  r=bb*sqrt(1+aa*aa); }

               e[i+1]=r;
               if (r == 0) {
                  d[i+1] -= p;
                  e[m]=0;
                  break;
               }
               s=f/r;
               c=g/r;
               g=d[i+1]-p;
               r=(d[i]-g)*s+2*c*b;
               d[i+1]=g+(p=s*r);
               g=c*r-b;
               for (k=0;k<n;k++) {
                  f=z[k*n+i+1];
                  z[k*n+i+1]=s*z[k*n+i]+c*f;
                  z[k*n+i]=c*z[k*n+i]-s*f;
               }
            }
            if (r == 0 && i >= j) continue;
            d[j]-=p; e[j]=g; e[m]=0;
         }
      } while (m != j);
   }
   return(status);
}
int sp_char(char c){
  return((c == '(' || c == ')' || c == ',' || c == ':' || c == ';'));
}
void ch_name(int lstring, char *tstring, int i){ 
  /* change name given by the first lstring chars of tstring to integer label i */
  char label[3];
  int tstl, llen;
  
  /* shorten the string and get rid of the name */
  tstl = 0;
  while(tstring[tstl+lstring] != '\0'){
    tstring[tstl] = tstring[tstl+lstring];
    tstl++;
  }
  tstring[tstl] = '\0';
  
  sprintf(label, "%i", i); /* i to char label */
  llen = strlen(label);   /* number of digits */
  
  /* make room for integer label; more efficient: combine this and shorten above */
  while(tstl >= 0){
    tstring[tstl+llen] = tstring[tstl];
    tstl--;
  }
  /* place the integer label where name used to be */
  for(tstl = 0; tstl < llen; tstl++)
    tstring[tstl] = label[tstl];
}
int has_bl(char *tstring)
{
  while(*tstring != ';')
    if(*tstring == ':') /* signifies start of bl */
      return(1);
    else
      tstring++;
  return(0); /* must not be bl if we got this far */
}
void ch_labels_nbl(int ntaxa, char *tstring, char names[][11]){
  /* change all names in tstring to integer labels (no branch lengths)*/
  char taxname[11];
  int ltstring, i;

  while(*tstring != ';'){
    while(*tstring == ')' || *tstring == '(' || *tstring == ','){
      tstring++;
    }
    if(*tstring == ';') break;  /* end of tstring, no more names */

    ltstring = 0; /* beginning of new name */
    while(*tstring != ')' && *tstring != '(' && *tstring != ','){
      taxname[ltstring] = *tstring;
      tstring++;
      ltstring++;
    }

    for(i=ltstring; i < 10; i++) taxname[i] = ' '; /* pad w/ blanks */
    taxname[10] = '\0'; /* name is in taxname */
    for(i = 0; i < ltstring; i++) tstring--; /* go to start of name */
    
    for(i = 0; i < ntaxa; i++){            /* check which taxa it is */
      if (strncmp(taxname, names[i], 10) == 0){ /* i is label for taxa */
	ch_name(ltstring, tstring, i);	  
	break;
      }
    }
    /* go to end of name */
    while(*tstring != '(' && *tstring != ')' && *tstring != ',') tstring++;
  }
}
void ch_labels(int ntaxa, char *tstring, char names[][11]){
  /* change all names in tstring to integer labels */
  char taxname[11];
  int lstring, i;
  
  while(*tstring != ';'){
    while(*tstring != ':' && *tstring != ';'){ /* name always before : */
      tstring++;
    }
    if (*tstring == ';') break;  /* end of tstring, no more names */
    tstring--; lstring = 0;      /* end of a possible name */
    while(!sp_char(*tstring)){   /* find the beginning */
      tstring--; lstring++;
    }
    if (*tstring == '(' || *tstring == ','){ /* condition for a name */
      tstring++;
      strncpy(taxname, tstring, lstring);    /* name is in taxname */
      for(i=lstring; i < 10; i++) taxname[i] = ' '; /* pad w/ blanks */
      taxname[10] = '\0';

/*       for(i = 0; i < lstring; i++) putchar(taxname[i]); printf("\n");  */
      for(i = 0; i < ntaxa; i++){            /* check which taxa it is */
/*  	printf("%s\n", names[i]);  */
	if (strncmp(taxname, names[i], 10) == 0){ /* i is label for taxa */
	  ch_name(lstring, tstring, i);	  
	  break;
	}
      }
/*       printf("%s\n\n", tstring); */ 
    }
    while(*tstring != ':') tstring++; /* get back to : */
    tstring++; /* process the rest of string */
  }
}
int is_wsp(char c){
  if(c==' ' || c=='\t' || c=='\n' || c=='\t') return(1);
  return(0);
}
void rmblntree(char *tstring, int rwsp){
  /* I: wsp=1 => remove whitespace */
  int s=0,e=0;
  while(tstring[e]!=';'){
    if(tstring[e]==':')
      while(tstring[e]!=',' && tstring[e]!=')') e++;
    if(!rwsp || (tstring[e]!=' ' && tstring[e]!='\t'&& tstring[e]!='\r' && 
		 tstring[e]!='\n'))
      tstring[s++]=tstring[e];
    e++;
  }
  tstring[s++]=';';
  tstring[s]='\0';
}
void nj(int numsp, double *d, double *ntree)
{
  /* only the upper triangle of d used and changed*/
  int i,j,k,njoin,m,rm,lm;
  double *r,dmin,t1,t2;

  r = cvec(numsp);
  m = numsp;

  for(njoin = 0; njoin < numsp-2; njoin++){
    for(i = 0; i < m; i++){
      r[i] = 0.0;
      for(j = 0; j < i; j++) r[i] += d[j+i*numsp];
      for(j = (i+1); j < m; j++) r[i] += d[i+j*numsp];
    }
    rm = 0; lm = 1;
    dmin = d[numsp] -(r[1] + r[0])/(m-2);
    for(i = 0; i < m; i++)
      for(j = (i+1); j < m; j++)
	  if ((d[i+j*numsp] - (r[i] + r[j])/(m - 2)) < dmin){
	    rm = i;
	    lm = j;
	    dmin = d[i+j*numsp]- (r[i] + r[j])/(m - 2);
/* 	    printf("%i %i %i %f\n", njoin, i, j, dmin); */
      }
    
    ntree[njoin*4] = rm;
    ntree[1+njoin*4] = lm;
    t1 = 0.5*(d[rm+lm*numsp] + (r[rm] - r[lm])/(m-2));
    t2 = 0.5*(d[rm+lm*numsp] + (r[lm] - r[rm])/(m-2));
    if (t1 > 0 & t2 > 0){
      ntree[2+njoin*4] = t1;
      ntree[3+njoin*4] = t2;
    }
    if (t1 > 0 & t2 < 0){
      ntree[2+njoin*4] = t1+t2;
      ntree[3+njoin*4] = 0;
    }
    if (t1 < 0 & t2 > 0){
      ntree[2+njoin*4] = 0;
      ntree[3+njoin*4] = t1+t2;
    }
    
    for(k = 0; k < rm; k++)
      d[k+rm*numsp] = (d[k+rm*numsp] + d[k+lm*numsp] - d[rm+lm*numsp])/2.0;
    for(k = (rm+1); k < lm; k++)
      d[rm+k*numsp] = (d[rm+k*numsp] + d[k+lm*numsp] - d[rm+lm*numsp])/2.0;
    for(k = (lm+1); k < m; k++)
      d[rm+k*numsp] = (d[rm+k*numsp] + d[lm+k*numsp] - d[rm+lm*numsp])/2.0;
    m--;
    for(i = 0; i < lm; i++)
      for(j = lm; j < m; j++)
	d[i+j*numsp] = d[i+(j+1)*numsp];
    for(i = lm; i < m; i++)
      for(j = (i+1); j < m; j++)
	d[i+j*numsp] = d[(i+1)+(j+1)*numsp];
  }
  ntree[(numsp-2)*4] = 0;
  ntree[1+(numsp-2)*4] = 1;
  ntree[2+(numsp-2)*4] = d[numsp];
  ntree[3+(numsp-2)*4] = 0.0;
  free(r);
}
void nju(int numsp, double *d, double *ntree){
  double *otree;
  otree = cvec(numsp*4);
  nj(numsp, d, otree);
  ou2nu(numsp,otree,ntree);
  ntree[2+(numsp-2)*4] += ntree[3+(numsp-2)*4];
  ntree[3+(numsp-2)*4] = 0.0;
  free(otree);
}
void split_leading1(int numsp, int *split){
  int i;
  if (split[0] == 1) return;
  for(i = 0; i < numsp; i++) split[i] = (split[i] == 1)?0:1;
}
void utreec2splits(int numsp, double *utreec, int *splits){
  int i,r,l,split;
  /* ith column gives split for numsp+ith branch */
  for(split = 0; split < numsp-3; split++){
    r = (int) utreec[split*4]; l = (int) utreec[1+split*4];
    if (r < numsp){
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = 0;
      splits[r+split*numsp] = 1;
    }
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = splits[i+(r-numsp)*numsp];
    if (l < numsp)
      splits[l+split*numsp] = 1;
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] += splits[i+(l-numsp)*numsp];
  }
  for(split = 0; split < numsp-3; split++)
    split_leading1(numsp,&splits[split*numsp]);
}
int ivsame(int n, int *x, int *y){
  int i;
  for(i = 0; i < n; i++)
    if (x[i] != y[i]) return(0);
  return(1);
}
int up_usplitr(int numsp, int *nspl, int *split, int *usplits){
  int ispl,i; 
 for(ispl = 0; ispl < (*nspl); ispl++)
   if (ivsame(numsp,split,&usplits[ispl*numsp])) return(ispl);
 for(i = 0; i < numsp; i++) usplits[i+(*nspl)*numsp] = split[i];
 (*nspl) += 1;
 return(((*nspl) - 1));
}
int split_sum(int numsp, int *split, int *row_idx){
  int i,ssum;
  ssum = 0;
  for(i = 0; i < numsp; i++) 
    if (row_idx[i] >= 0) ssum += split[i];
  return(ssum);
}
void up_utree(int numsp, int join, int ssum, int *splits, int *row_idx, 
	      double *utreec){
  int pair_idx,r,l,i,np,pair[2];
  if (ssum == (numsp-join-2)) pair_idx = 0; /* get pair for join */
  if (ssum == 2) pair_idx = 1;
  np = 0; 
  for(i = 0; i < numsp; i++) 
    if (row_idx[i] >= 0 && splits[i] == pair_idx){
      pair[np] = row_idx[i];
      np++;
      if (np == 1) row_idx[i] = -1; /* update row */
      if (np == 2){
	row_idx[i] = join+numsp; /* update row */
	break;
      }
    }
  r = (pair[0] < pair[1])?pair[0]:pair[1];
  l = (pair[0] < pair[1])?pair[1]:pair[0];
  utreec[0] = r; utreec[1] = l; 
}
void final_joins(int numsp, int *row_idx, double *utreec){
  int isp,i,lab[3];

  utreec[5] = 2*numsp-3;
  isp = 0;
  for(i = 0; i < numsp; i++)
    if (row_idx[i] >= 0){
      lab[isp] = row_idx[i];
      isp++;
      if (isp > 2) break;
    }
  utreec[0] = (lab[0] < lab[1])?lab[0]:lab[1];
  utreec[1] = (lab[0] < lab[1])?lab[1]:lab[0];
  utreec[4] = lab[2];
}
void splits2tree_bl(int numsp, int nspl, int *tree, int *usplits, double *bl, 
		    double *utreec){
  int i,j,r,ssum,join,split,*row_idx,*col_idx;
  double *blj;

  row_idx = (int *) malloc((size_t) numsp*sizeof(int));
  col_idx = (int *) malloc((size_t) nspl*sizeof(int));
  blj = (double *) malloc((size_t) (numsp-3)*sizeof(double));

  for(i = 0; i < numsp; i++) row_idx[i] = i;
  for(i = 0; i < nspl; i++) col_idx[i] = 1;
  for(i = 0; i < numsp-3; i++) col_idx[tree[i]] = 0;

  for(join = 0; join < numsp-3; join++){

    for(split = 0; split < nspl; split++){ /* find cherry */
      if (col_idx[split] == 0)
	ssum = split_sum(numsp,&usplits[split*numsp],row_idx);
      if (col_idx[split] == 0 && (ssum == 2 || ssum == (numsp-join-2))) break;
    }

    col_idx[split] = 1;
    up_utree(numsp,join,ssum,&usplits[split*numsp],row_idx,&utreec[join*4]);
    blj[join]=bl[split+numsp];
  }

  final_joins(numsp, row_idx, &utreec[(numsp-3)*4]);

  for(i = 0; i < numsp-1; i++)
    for(j = 0; j < 2; j++){
      if ((r = (int) utreec[j+i*4]) >= numsp && r < 2*numsp-3){
	utreec[j+2+i*4] = blj[r-numsp];
      }
      if (r < numsp) utreec[j+2+i*4] = bl[r];
    }

/*   printf("\n"); */
/*   pr_utreec(stdout,numsp,utreec); */
/*   printf("\n"); */
  free(row_idx); free(col_idx); free(blj);
}
void splits2tree_nobp(int numsp, int nspl, int *tree, int *usplits, 
		      double *utreec){
  int i,j,jup,ssum,join,split,*row_idx,*col_idx;

  row_idx = (int *) malloc((size_t) numsp*sizeof(int));
  col_idx = (int *) malloc((size_t) nspl*sizeof(int));

  for(i = 0; i < numsp; i++) row_idx[i] = i;
  for(i = 0; i < nspl; i++) col_idx[i] = 1;
  for(i = 0; i < numsp-3; i++) col_idx[tree[i]] = 0;

  for(join = 0; join < numsp-3; join++){

    for(split = 0; split < nspl; split++){ /* find cherry */
      if (col_idx[split] == 0)
	ssum = split_sum(numsp,&usplits[split*numsp],row_idx);
      if (col_idx[split] == 0 && (ssum == 2 || ssum == (numsp-join-2))) break;
    }

    col_idx[split] = 1;
    up_utree(numsp,join,ssum,&usplits[split*numsp],row_idx,&utreec[join*4]);
  }

  final_joins(numsp, row_idx, &utreec[(numsp-3)*4]);

  for(i = 0; i < numsp-1; i++){
    jup = (i < numsp-2)?2:1;
    for(j = 0; j < jup; j++) utreec[j+2+i*4] = 0.1;
  }
  utreec[3+(numsp-2)*4]=0;

/*   printf("\n"); */
/*   pr_utreec(stdout,numsp,utreec); */
/*   printf("\n"); */
  free(row_idx); free(col_idx); 
}
void rmexspce(int numsp, char names[][11]){
  int i,j;
  for(i = 0; i < numsp; i++)
    for(j = 9; j >= 0; j--)
      if (names[i][j] != ' ' && names[i][j] != '\0' &&  names[i][j] != '\t' && names[i][j] != '\r'){
	names[i][j+1] = '\0';
	break;
      }
}
int ndig_intpart(double f){
  int d=1,ndigits=0;
  while(((int) f/d) > 0){
    d *= 10;
    ndigits++;
  }
  if (ndigits == 0) ndigits=1;
  return(ndigits);
}
char *ut2in(int numsp, double *utreec, char names[][11], char **ilabels){
  char **tstring,*tstr;
  int i,alen,len,r,l;
  double x;

  tstring = (char **) malloc((size_t) (2*numsp-2)*sizeof(char *));

  rmexspce(numsp,names);
  for(i = 0; i < numsp; i++){
    len = lstring(names[i])+1;
    tstring[i] = (char *) malloc((size_t) len*sizeof(char));
    strcpy(tstring[i], names[i]);
  }

  for(i = 0; i < (numsp-3); i++){
    /* memory allocation */
    r = utreec[i*4]; l = utreec[1+i*4];
    len = lstring(tstring[r]) + lstring(tstring[l]) +
      ndig_intpart(utreec[2+i*4]) + ndig_intpart(utreec[3+i*4]) + 18;
    if (r >= numsp) len += lstring(ilabels[r-numsp]);
    if (l >= numsp) len += lstring(ilabels[l-numsp]);
    tstring[i+numsp] = (char *) malloc((size_t) len*sizeof(char));
    /* create subtree */
    if (r >= numsp && l >= numsp)
      alen=sprintf(tstring[i+numsp],"(%s%s:%.5f,%s%s:%.5f)",tstring[r],
	      ilabels[r-numsp],utreec[2+i*4],tstring[l],ilabels[l-numsp],
	      utreec[3+i*4]);
    if (r >= numsp && l < numsp)
      alen=sprintf(tstring[i+numsp],"(%s%s:%.5f,%s:%.5f)",tstring[r],
	      ilabels[r-numsp],utreec[2+i*4],tstring[l],utreec[3+i*4]);
    if (r < numsp && l >= numsp)
      alen=sprintf(tstring[i+numsp],"(%s:%.5f,%s%s:%.5f)",tstring[r],
	     utreec[2+i*4],tstring[l],ilabels[l-numsp],utreec[3+i*4]);
    if (r < numsp && l < numsp)
      alen=sprintf(tstring[i+numsp],"(%s:%.5f,%s:%.5f)",tstring[r],
	      utreec[2+i*4],tstring[l],utreec[3+i*4]);
    /* printf("%i %i: %s\n",alen+1,i+numsp,tstring[i+numsp]); */
  }
  /* PHYLIP requires trifurcation */
  /* memory allocation */
  r = utreec[(numsp-3)*4]; l = utreec[1+(numsp-3)*4];
  len = lstring(tstring[r]) + lstring(tstring[l]) + 16 +
    ndig_intpart(utreec[2+(numsp-3)*4])+ndig_intpart(utreec[3+(numsp-3)*4]);
  if (r >= numsp) len += lstring(ilabels[r-numsp]);
  if (l >= numsp) len += lstring(ilabels[l-numsp]);
  tstring[2*numsp-3] = (char *) malloc((size_t) len*sizeof(char));
  /* create subtree */
  i=numsp-3;
  if (r >= numsp && l >= numsp)
    alen=sprintf(tstring[i+numsp],"%s%s:%.5f,%s%s:%.5f",tstring[r],
	    ilabels[r-numsp],utreec[2+i*4],tstring[l],ilabels[l-numsp],
	    utreec[3+i*4]);
  if (r >= numsp && l < numsp)
    alen=sprintf(tstring[i+numsp],"%s%s:%.5f,%s:%.5f",tstring[r],
	    ilabels[r-numsp],utreec[2+i*4],tstring[l],utreec[3+i*4]);
  if (r < numsp && l >= numsp)
    alen=sprintf(tstring[i+numsp],"%s:%.5f,%s%s:%.5f",tstring[r],
	    utreec[2+i*4],tstring[l],ilabels[l-numsp],utreec[3+i*4]);
  if (r < numsp && l < numsp)
    alen=sprintf(tstring[i+numsp],"%s:%.5f,%s:%.5f",tstring[r],
	    utreec[2+i*4],tstring[l],utreec[3+i*4]);
  /* printf("%i %i %i %i %i: %s\n",r,l,len,alen+1,2*numsp-3,tstring[2*numsp-3]); */
  /* memory allocation for final string */
  r = utreec[(numsp-2)*4]; l = utreec[1+(numsp-2)*4];
  r = (r < l)?r:l;
  x=utreec[2+(numsp-2)*4]+utreec[3+(numsp-2)*4];
  len = lstring(tstring[2*numsp-3]) + lstring(tstring[r]) + 12 + 
    ndig_intpart(x);
  if (r >= numsp) len += lstring(ilabels[r-numsp]);
  tstr=(char *) malloc((size_t) len*sizeof(char));
  /* tree */
  if (r >= numsp)
    alen=sprintf(tstr,"(%s,%s%s:%.5f);",tstring[2*numsp-3],tstring[r],
	    ilabels[r-numsp],x);
  if (r < numsp)
    alen=sprintf(tstr,"(%s,%s:%.5f);",tstring[2*numsp-3],tstring[r],x);
  tstr[len-1] = '\0';
  /* printf("%i %i %i %s\n",r,len,alen+1,tstr); */
  for(i = 0; i < 2*numsp-2; i++) free(tstring[i]);
  free(tstring);
  return(tstr);
}
void ut2n(int numsp, double *utreec, char names[][11], char *tstr){
  char **tstring;
  int i,len,r,l;

  tstring = (char **) malloc((size_t) (2*numsp-2)*sizeof(char *));

  for(i = 0; i < numsp; i++){
    len = lstring(names[i]);
    tstring[i] = (char *) malloc((size_t) len*sizeof(char));
    strcpy(tstring[i], names[i]);
    /* remove trailing blanks */
    while(tstring[i][len-1]==' ') len--;
    tstring[i][len]='\0';
  }

  for(i = 0; i < (numsp-3); i++){
    r = utreec[i*4]; l = utreec[1+i*4];
    len = lstring(tstring[r]) + lstring(tstring[l])+23; /* space for branch lengths in the 100s */
    tstring[i+numsp] = (char *) malloc((size_t) len*sizeof(char));
    sprintf(tstring[i+numsp],"(%s:%.5f,%s:%.5f)",tstring[r],utreec[2+i*4],tstring[l],
	    utreec[3+i*4]);
  }
  /* PHYLIP requires trifurcation */
  r = utreec[(numsp-3)*4]; l = utreec[1+(numsp-3)*4];
  len = lstring(tstring[r]) + lstring(tstring[l]) + 21;
  tstring[2*numsp-3] = (char *) malloc((size_t) len*sizeof(char));
  sprintf(tstring[2*numsp-3],"%s:%.5f,%s:%.5f",tstring[r],utreec[2+(numsp-3)*4],tstring[l],
	    utreec[3+(numsp-3)*4]);
  r = utreec[(numsp-2)*4]; l = utreec[1+(numsp-2)*4];
  r = (r < l)?r:l;
  sprintf(tstr,"(%s,%s:%.5f);",tstring[2*numsp-3],tstring[r],
	    utreec[2+(numsp-2)*4]+utreec[3+(numsp-2)*4]);
  len = lstring(tstr);
  tstr[len] = '\0';
  for(i = 0; i < 2*numsp-2; i++) free(tstring[i]);
  free(tstring);
}
void ut2nt(int numsp, double *utreec, char names[][11], char *tstr){
  char **tstring;
  int i,len,r,l;

  tstring = (char **) malloc((size_t) (2*numsp-2)*sizeof(char *));

  for(i = 0; i < numsp; i++){
    len = lstring(names[i]);
    tstring[i] = (char *) malloc((size_t) len*sizeof(char));
    strcpy(tstring[i], names[i]);
    /* remove trailing blanks */
    while(tstring[i][len-1]==' ') len--;
    tstring[i][len]='\0';
  }

  for(i = 0; i < (numsp-3); i++){
    r = utreec[i*4]; l = utreec[1+i*4];
    len = lstring(tstring[r]) + lstring(tstring[l])+4; 
    tstring[i+numsp] = (char *) malloc((size_t) len*sizeof(char));
    sprintf(tstring[i+numsp],"(%s,%s)",tstring[r],tstring[l]);
  }
  /* PHYLIP requires trifurcation */
  r = utreec[(numsp-3)*4]; l = utreec[1+(numsp-3)*4];
  len = lstring(tstring[r]) + lstring(tstring[l]) +2;
  tstring[2*numsp-3] = (char *) malloc((size_t) len*sizeof(char));
  sprintf(tstring[2*numsp-3],"%s,%s",tstring[r],tstring[l]);
  r = utreec[(numsp-2)*4]; l = utreec[1+(numsp-2)*4];
  r = (r < l)?r:l;
  sprintf(tstr,"(%s,%s);",tstring[2*numsp-3],tstring[r]);
  len = lstring(tstr);
  tstr[len] = '\0';
  for(i = 0; i < 2*numsp-2; i++) free(tstring[i]);
  free(tstring);
}
void simp_qr(int n, int p, double *X, double *Q, double *R){
  int info,i,j;
  double *tau,*work;

  tau = (double *) malloc(p*sizeof(double));
  work = (double *) malloc(n*sizeof(double));
  memcpy(Q,X,n*p*sizeof(double));
  dgeqrf_(&n,&p,Q,&n,tau,work,&n,&info);
  for(i = 0; i < p*p; i++) R[i]=0.0;
  for(i = 0; i < p; i++)
    for(j = i; j < p; j++)
	  R[i+j*p]=Q[i+j*n];
  dorgqr_(&n,&n,&p,Q,&n,tau,work,&n,&info);
  free(tau); free(work);
}
double lsqr(int n, int p, int est_beta, double *Q, double *R, double *y, 
	    double *beta){
  int i,j;
  double ss = 0.0,z;
  for(i = 0; i < (n-p); i++){
    z = 0.0;
    for(j = 0; j < n; j++) z += Q[j+(i+p)*n]*y[j];
    ss += z*z;
  }
  if(est_beta){
    for(i = p-1; i >= 0; i--){
      beta[i]=0.0;
      for(j = 0; j < n; j++)  beta[i] += Q[j+i*n]*y[j];
      for(j = i+1; j < p; j++) beta[i] -= R[i+j*p]*beta[j];
      beta[i] /= R[i+i*p];
    }
  }
  return(ss);
}
void getspl_spr(int ntaxa, int i, int **s, int *tree, int *usplits, int *terms){
  if (i < ntaxa)
    *s = &terms[i*ntaxa];
  if (i >= ntaxa)
    *s = &usplits[tree[i-ntaxa]*ntaxa];
}
void getAD(int ntaxa, int *s1, int *s2, int *dA, int *dD){
  /* AD = null <==> sum_i d_i^A d_i^D = 0 */
  int i,ds;
  memcpy(dA,s1,ntaxa*sizeof(int));
  memcpy(dD,s2,ntaxa*sizeof(int));
  ds = 0;
  for(i = 0; i < ntaxa; i++) ds += dA[i]*dD[i];
  if (ds == 0) return;
  ds = 0;
  for(i = 0; i < ntaxa; i++) dD[i] = 1 - dD[i];
  for(i = 0; i < ntaxa; i++) ds += dA[i]*dD[i];
  if (ds == 0) return;
  ds = 0;
  for(i = 0; i < ntaxa; i++) dA[i] = 1 - dA[i];
  for(i = 0; i < ntaxa; i++) ds += dA[i]*dD[i];
  if (ds == 0) return;
  for(i = 0; i < ntaxa; i++) dD[i] = 1 - dD[i];
}
int checkC(int ntaxa, int *dA, int *dD, int *s){
  /* SW=W <==> sum_i d_i^S d_i^W = sum_i d_i^W
   *      <==> sum_i d_i^W (1-d_i^S) = 0 
   *
   * Return: -1 if not a split in C
   *          0 if E_0 D = D
   *          1 if E_1 D = D
   */
  int i,AE1,AE0,DE0,DE1;

  AE1 = 0;
  for(i = 0; i < ntaxa; i++) AE1 += dA[i]*(1-s[i]);
  if (AE1 == 0){
    DE0=0;
    for(i = 0; i < ntaxa; i++) DE0 += dD[i]*s[i];
    if (DE0 == 0) return(0);
  }
  if (AE1 != 0){
    AE0 = 0;
    for(i = 0; i < ntaxa; i++) AE0 += dA[i]*s[i];
    if (AE0 == 0){
      DE1 = 0;
      for(i = 0; i < ntaxa; i++) DE1 += dD[i]*(1-s[i]);
      if (DE1 == 0) return(1);
    }
  }
  return(-1);
}
void upsplit_spr(int ntaxa, int Ei, int *dA, int *s, int *sn){
  int j;
  for(j = 0; j < ntaxa; j++)
    if (dA[j] || s[j] == Ei) 
      sn[j] = 1;
    else
      sn[j] = 0;
  split_leading1(ntaxa,sn);
}
void uptrees(int ntaxa, int *tree, int *ntrees, int *trees){
  int i;
  for(i = 0; i < *ntrees; i++)
    if (ivsame(ntaxa-3,tree,&trees[i*(ntaxa-3)])){
      memcpy(&trees[(*ntrees)*(ntaxa-3)],tree,(ntaxa-3)*sizeof(int));
      (*ntrees)++;
    }
}
int is_newsf(int nsplits, int s, int *tree){
  int i;
  if (nsplits == 0) return(1);
  for(i = 0; i < nsplits; i++) 
    if (tree[i] == s) return(0);
  return(1);
}
int is_termf(int ntaxa, int *sn){
  int i,n0=0,n1=0;
  for(i = 0; i < ntaxa; i++)
    if (sn[i] == 0)
      n0++;
    else
      n1++;
  if (n0 == 1 || n1 == 1) 
    return(1);
  else
    return(0);
}
void sprtrees(int ntaxa, int *tree, int *usplits, int *nspl, int *ntrees, 
	     int *trees){
  int i,i1,i2,*s1,*s2,si,*dA,*dD,Ei,*sn,cspl,*treen,*terms,is_new,tr;
  
  dA = (int *) malloc((size_t) ntaxa*sizeof(int));
  dD = (int *) malloc((size_t) ntaxa*sizeof(int));
  sn = (int *) malloc((size_t) ntaxa*sizeof(int));
  treen = (int *) malloc((size_t) (ntaxa-3)*sizeof(int));
  terms = (int *) malloc((size_t) ntaxa*ntaxa*sizeof(int));

  for(i = 0; i < ntaxa*ntaxa; i++) terms[i]=0; /* violates s[0]=1 convention */
  for(i = 0; i < ntaxa; i++) terms[i+i*ntaxa]=1;
  
  for(i1 = 0; i1 < 2*ntaxa-3; i1++)/* loop through pairs of splits in spr */
    for(i2 = 0; i2 < 2*ntaxa-3; i2++){
      if (i1 != i2){
	getspl_spr(ntaxa,i1,&s1,tree,usplits,terms); /* splits s1 and s2 */
	getspl_spr(ntaxa,i2,&s2,tree,usplits,terms);
      
	getAD(ntaxa,s1,s2,dA,dD);
	/* for(i = 0; i < ntaxa; i++) printf("%i", dA[i]); printf("\n"); */
/* 	for(i = 0; i < ntaxa; i++) printf("%i", dD[i]); printf("\n"); */

	/* printf("usplits\n"); */
/* 	for(j = 0; j < *nspl; j++){ */
/* 	  for(i = 0; i < ntaxa; i++) printf("%i",usplits[i+j*ntaxa]); */
/* 	  printf("\n"); */
/* 	} */
/* 	printf("usplits\n"); */
      
	cspl = 0; /* update splits */
	/* add the AD split */
	for(i = 0; i < ntaxa; i++) sn[i] = (dA[i] || dD[i])?1:0;
	split_leading1(ntaxa,sn);
	if (!is_termf(ntaxa,sn))
		treen[cspl++] = up_usplitr(ntaxa,nspl,sn,usplits);
/* 	printf("%i %i %i %i\n", i1, i2, *nspl, *ntrees); */
	/* for(i = 0; i < ntaxa; i++) printf("%i", sn[i]); printf("\n"); */
/* 	printf("%i\n",cspl); */
	
	for(si = 0; si < ntaxa-3; si++){ /* update other splits */
	  if (si != i1-ntaxa && si != i2-ntaxa){ /* not splits in spr */
	    Ei = checkC(ntaxa,dA,dD,&usplits[tree[si]*ntaxa]);   
	    /* printf("%i\n",Ei); */
	    if (Ei >= 0){
	      upsplit_spr(ntaxa,Ei,dA,&usplits[tree[si]*ntaxa],sn);
	      /* for(i = 0; i < ntaxa; i++) printf("%i", sn[i]); printf("\n"); */
	      if (!is_termf(ntaxa,sn)){
		tr = up_usplitr(ntaxa,nspl,sn,usplits);
		if(is_newsf(cspl,tr,treen)) treen[cspl++] = tr;
	      }
	    }
	    if (Ei < 0) 
	      if(is_newsf(cspl,tree[si],treen)){ 
		treen[cspl++] = tree[si];
		/* printf("%i\n",tree[si]); */
	      }
	  }
	}
	if (i1 >= ntaxa) treen[cspl++] = tree[i1-ntaxa];
	if (i2 >= ntaxa) treen[cspl++] = tree[i2-ntaxa];
	/* printf("%i\n",cspl); */
	isort(ntaxa-3,treen);
	/* for(i = 0; i < ntaxa-3; i++) printf("%i",treen[i]); printf("\n"); */
/* 	for(i = 0; i < ntaxa-3; i++) printf("%i",trees[i]); printf("\n"); */
	is_new = 1;
	for(i = 0; i < *ntrees; i++)
	  if (ivsame(ntaxa-3,treen,&trees[i*(ntaxa-3)])){
	    is_new = 0;
	    break;
	  }
	/* printf("%i\n",is_new); */
	if(is_new){ 
	  memcpy(&trees[(*ntrees)*(ntaxa-3)],treen,(ntaxa-3)*sizeof(int));
	  (*ntrees)++;
	}
	/* printf("%i %i %i\n-------\n", i1, i2, *ntrees); */
      }}

  free(dA); free(dD); free(sn); free(treen); free(terms);

}
int rfone(int ntaxa, int *t1, int *t2){
  int jo=0,i,j,rf=0;

  for(i = 0; i < ntaxa-3; i++){
    /* find first j w/ t1[i]<= t1[j] */
    for(j = jo; j < ntaxa-3; j++)
      if(t1[i] <= t2[j]) break;
    /* j==ntaxa-3 => couldn't find one
     * => t1[i] > all t2[j]
     * since t1[ntaxa-4] > ... > t1[i] > all t2[j], can update rf and exit */
    if(j==ntaxa-3)return(rf+ntaxa-4-i+1);
    /* j<ntaxa-3 & t1[i]<t2[j] => 
     *         t1[i] > t2[j-1] > ... > t2[0]
     *               o/w j would not be first j w/ t[i] <= t[j]
     *        &t1[i] < t2[j] < ... t2[ntaxa-3] => no match */
    if(j<ntaxa-3 && t1[i]<t2[j]) rf++;
    /* since t1[i] > t2[j-1] 
     *  then t1[ntaxa-3] > ... > t1[i] > t2[j-1] > ... t2[0]
     *  so all further searches can be restricted to j >= j */
    jo=j;
  }
  return(rf);
}
int rfdist(int ntaxa, int *t1, int *t2){
  int rf;
 
  rf = rfone(ntaxa,t1,t2);
  rf += rfone(ntaxa,t2,t1);
  return(rf);
}
int rfonef(int ntaxa, int ls, int it, int *t){
  int rf=0;
  while(it < ntaxa-3){
    if(t[it] < ls){
      it++;
      rf++;
    }
    else if(t[it] == ls){
      return(rf+ntaxa-4-it);
    }
    else{
      return(rf+ntaxa-3-it);
    }
  }
  return(rf);
}
int spr_search(int ntaxa, int ntrees, int bl, double *utreec, double *fnval, 
	       int *stop){
  static int initialize=1,ctree=0,ntrees_avail=0,opt_tree=0;
  static int ntrees_alloc,nspl_alloc,nspl;
  static int *usplits,*trees;
  static double *fvals,fopt,*blm;
  int i,j,new_opt,nspln,ntreesn,*o,*tmp_trees,*rf,pr=0;
  
  /* initialization */
  if(initialize){
    nspl_alloc = ntaxa-3; 
    nspl = ntaxa-3;
    ntrees_alloc = 1;
    usplits = (int *) malloc(nspl_alloc*ntaxa*sizeof(int));
    fvals = (double *) malloc(ntrees_alloc*sizeof(double));
    trees = (int *) malloc(ntrees_alloc*(ntaxa-3)*sizeof(int));
    if(bl) 
      blm = (double *) malloc(ntrees_alloc*(2*ntaxa-3)*sizeof(double));
    utreec2splits(ntaxa,utreec,usplits);
    for(i = 0; i < ntaxa-3; i++) trees[i]=i;
    initialize = 0;
    return(ctree);
  }

  if(*stop == 1){
    if(ntrees == 1){
      *fnval = fopt;
      splits2tree_nobp(ntaxa,nspl,&trees[opt_tree*(ntaxa-3)],usplits,utreec);
      if(bl) bl2utreec(ntaxa,&blm[opt_tree*(2*ntaxa-3)],utreec);
    }
    if(ntrees > 1){
      o = (int *) malloc((size_t) ctree*sizeof(int));
      dorder(ctree,fvals,o);
      for(i = 0; i < ntrees; i++){
	splits2tree_nobp(ntaxa,nspl,&trees[o[i]*(ntaxa-3)],usplits,
			 &utreec[i*(ntaxa-1)*4]);
	if(bl) bl2utreec(ntaxa,&blm[o[i]*(2*ntaxa-3)],&utreec[i*(ntaxa-1)*4]);
	fnval[i]=fvals[o[i]];
      }
      free(o);
    }
    free(usplits); free(fvals); free(trees); 
    if(bl) free(blm);
    return(ctree);
  }

  /* an evaluation has been made */
  fvals[ctree]=*fnval; 
  if(bl) utreec2bl(ntaxa,utreec,&blm[ctree*(2*ntaxa-3)]);
/*   wrow(stdout,2*ntaxa-3,blm,1); */
  if(pr>=2){
    for(i = 0; i < ntaxa-3; i++) printf("%2i ",trees[i+ctree*(ntaxa-3)]);
    printf("%f\n",*fnval);
  }

  /* is it the best fnval so far? */
  new_opt=0;
  if(ctree==0) fopt = *fnval+1; /* new_opt for sure initially */
  if(*fnval < fopt){
    if(pr > 0){
      for(i = 0; i < ntaxa-3; i++) printf("%2i ",trees[i+ctree*(ntaxa-3)]);
      printf("%f\n",*fnval);
    }
    opt_tree = ctree;
    fopt = *fnval;
    new_opt=1;
  }

  /* if a new_opt, get all sprs for opt tree so far */
  if(new_opt){
    nspln = nspl+(2*ntaxa-3)*(2*ntaxa-4)*(ntaxa-3);
    ntreesn = ctree+1+(2*ntaxa-3)*(2*ntaxa-4);
    if(nspln > nspl_alloc){
      usplits = (int *) realloc(usplits,nspln*ntaxa*sizeof(int));
      nspl_alloc = nspln;
    }
    if(ntreesn > ntrees_alloc){
      trees = (int *) realloc(trees,ntreesn*(ntaxa-3)*sizeof(int));
      fvals = (double *) realloc(fvals,ntreesn*sizeof(double));
      if(bl)
	blm = (double *) realloc(blm,ntreesn*(2*ntaxa-3)*sizeof(double));
      ntrees_alloc = ntreesn;
    }
    ntrees_avail = ctree+1;
    sprtrees(ntaxa,&trees[ctree*(ntaxa-3)],usplits,&nspl,&ntrees_avail,trees);
    /* SPR trees sorted according to distance from optimal tree */
    if(pr >= 2){
      printf("%i trees evaluated, %i trees available, %i trees allocated\n",
	     ctree+1,ntrees_avail,ntrees_alloc);
      printf("%i splits, %i splits allocated\n",nspl,nspl_alloc);
    }
    if(pr >= 3){
      printf("SPR trees sorted according to distance from optimal tree\n");
    }
    if(ntrees_avail-ctree > 1){
      rf = (int *) malloc((ntrees_avail-ctree-1)*sizeof(int));
      tmp_trees=(int *) malloc((ntrees_avail-ctree-1)*(ntaxa-3)*sizeof(int));
      o = (int *) malloc((ntrees_avail-ctree-1)*sizeof(double));
      for(i=ctree+1; i<ntrees_avail; i++)
	rf[i-ctree-1]=rfdist(ntaxa,&trees[ctree*(ntaxa-3)],&trees[i*(ntaxa-3)]);
      iorder(ntrees_avail-ctree-1,rf,o);
      memcpy(tmp_trees,&trees[(ctree+1)*(ntaxa-3)],
	     (ntrees_avail-ctree-1)*(ntaxa-3)*sizeof(int));
      for(i=ctree+1; i<ntrees_avail; i++){
	memcpy(&trees[i*(ntaxa-3)],&tmp_trees[o[i-ctree-1]*(ntaxa-3)],
	       (ntaxa-3)*sizeof(int));
	if(pr >= 3){
	  for(j = 0; j < ntaxa-3; j++) printf("%2i ",trees[j+i*(ntaxa-3)]);
	  printf(": %i\n",rf[o[i-ctree-1]]);
	}
      }
      free(rf);
      free(o);
      free(tmp_trees);
    }
  }
    
  /* next tree */
  ctree++;
  if(ctree < ntrees_avail)
    splits2tree_nobp(ntaxa,nspl,&trees[ctree*(ntaxa-3)],usplits,utreec);
  if(ctree >= ntrees_avail) *stop = 1;
  return(ctree);
}
